!***** IBM/volfrac_ibm_cube
! NAME
!  volfrac_ibm_helix
! SYNOPSIS
!  subroutine volfrac_ibm_cube
! AUTHOR
! Kevin M. & Pierre E.
! CREATION DATE
!  25/04/2020
! DESCRIPTION
! digitizer for volumetric fraction of solid at the pressure center
!!! MODIFICATION HISTORY
! PARENTS
!   ibm
! CHILDREN
!!! ARGUMENTS
! SOURCE
!
subroutine volfrac_ibm_helix(mm,alpha_ibm,i_dir)
  !
  !=================
  !   DECLARATIONS
  !=================
  use mod_nslcom3d
  use mod_param_ibm
  use mod_com_ibm
  use mod_structure
  use mod_helix
  !
  !.. Implicit Declarations ..
  implicit none
  !
  !.. Formal Arguments ..
  integer, intent(in) :: mm
  integer, intent(in) :: i_dir

  real(kind=8), dimension(nv_pmax), intent(out) :: alpha_ibm
  !
  !  Local scalars
  integer :: i,j,k,l,li,lj,lk
  integer :: i_box,j_box,k_box
  real(kind=8) :: r,r1,r2,r3
  real(kind=8) :: alpha
  integer ::  nzz!, n_turns
 ! File read
  real(kind=8), dimension(:,:,:), allocatable :: map_ibm
  character (len=2) :: tempo
  integer :: lines,ifich,io
  integer :: indexi,indexj,indexk
  integer :: imin,imax,jmin,jmax,kmin,kmax
  character (len=8) :: num


  ! Build alpha_ibm
  real(kind=8) :: h_h ,a_h, a_c ,xcilindro , ycilindro, zcilindro
  real(kind=8) :: TT1, UU, TT1_c, UU_c
  integer :: k_ibm, j_ibm, i_ibm
  real(kind=8) :: xh, yh, zh, x_h_rot, y_h_rot, z_h_rot, x_c_rot , y_c_rot, z_c_rot
  real(kind=8) :: theta_t, theta_u, theta_u_c, theta_t_c, TT1_rot_heli
  real(kind=8) :: TT1_c_0, UU_c_0, a_c_0, xcilindro_0, ycilindro_0, zcilindro_0
  real(kind=8) :: x_rot_c_0, y_rot_c_0, z_rot_c_0, interp_dx_c, interp_dy_c, interp_dz_c
  real(kind=8) :: tauibm, cntr_dis_x, cntr_dis_y, cntr_dis_z
  real(kind=8) :: TT1_0, UU_0, a_h_0, xh_0, yh_0, zh_0
  real(kind=8) :: interp_dx, interp_dy, interp_dz, x_rot_0, y_rot_0, z_rot_0
  real(kind=8) :: x_esf_rot_0, y_esf_rot_0, z_esf_rot_0 ,a_esf_0, TT1_esf_0, UU_esf_0
  real(kind=8) :: x_esf_0, y_esf_0, z_esf_0, interp_dx_esf, interp_dy_esf, interp_dz_esf
  integer :: k_ibm_0, j_ibm_0, i_ibm_0, ii
  real(kind=8) :: ni, ntot
  ! Sigmoid Smooth
  real(kind=8) :: sigma_slope

  !.. Intrinsic Functions ..
  intrinsic dabs,dsqrt,dtanh
  !
  ! End of Declarations

  ! Fill alpha_ibm
  alpha_ibm(:)  = 0.d0

  ! Helix customization
  theta_t= n_turns*2*pi          !Helix Azimuth (define the position)
  theta_u= 2*pi                  !number of local rounds over point in a specific theta_t (2*pi = 1 revolution)
  h_h= l_h_helix/theta_t         !Helix height/(nrev*2pi) = lambda/(2*pi)
  cntr_dis_x= xi_p(mm)           !Helix center measured from the origin in x
  cntr_dis_y= yi_p(mm)           !Helix center measured from the origin in y
  cntr_dis_z= zi_p(mm)           !Helix center measured from the origin in z For axial movement 20 must be looped

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!----------- Smooth parameters with sigmoid function------------------------------!!!!!!!!!!
  !smooth_portion= 0.2           ! 20/100 portion to be smoothed
  !smoothslope=4.5                 !how fast the curve decrease
  ntot= 100.00
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!------------In case of a cilinder head-------------------------------------------!!!!!!!!!!
  ! Cilinder parameters
  !Long_c= n_turns*h_h*(2*pi)      !cilinder length
  !thick_a_c= 0.008                !cilinder radius
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  allocate (map_ibm(nx, ny, nz))
  map_ibm=0

  if (jrot_p(mm)==2) then     ! imov_p
  TT1_rot_heli= npt*dt*(2.0*pi)*frecuencia_rot ! 2pi == 1Hz
  else
  TT1_rot_heli=0
  end if

      ! i_dir = 0 Calculation at the center of the pressure cell
      ! i_dir = 1 Calculation at the center of the velocity cell U
      ! i_dir = 2 Calculation at the center of the velocity cell V
      ! i_dir = 3 Calculation at the center of the velocity cell W

  imin=nx ;   imax=0
  jmin=ny ;   jmax=0
  kmin=nz ;   kmax=0
 ! Compute alpha_ibm
 !!!!!!!!!!!!!!!!!!!!!!!!---------------- Regular Mesh  ----------------------------!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

if (Block_helix_irregular==0) then
  do lk=1,(nz+1)
    TT1=(real(lk)-1.0)*theta_t/(nz)
    TT1_0=(real(lk)-2.0)*theta_t/(nz)

    do lj=1,(ny+1)
      UU=(real(lj)-1.0)*theta_u/(ny)
      UU_0=(real(lj)-2.0)*theta_u/(ny)

     do li=1,(nx+1)
        a_h=(real(li)-1.0)*2.0*thick_a/(nx)
        a_h_0=(real(li)-2.0)*2.0*thick_a/(nx)

        xh = RR*sin(TT1) - a_h*sin(TT1)*cos(UU) - a_h*h_h*cos(TT1)*sin(UU)
        yh = RR*cos(TT1) - a_h*cos(TT1)*cos(UU) + a_h*h_h*sin(TT1)*sin(UU)
        zh = h_h*(TT1)+a_h*RR*sin(UU)/(sqrt((RR*RR)+(h_h*h_h)))
        !!!! Rotation
        x_h_rot=(-xh*cos(TT1_rot_heli) + yh*sin(TT1_rot_heli));  !Rotation Matrix application "-" == conter-clockwise
        y_h_rot=xh*sin(TT1_rot_heli) + yh*cos(TT1_rot_heli);
        z_h_rot=zh;

        i_ibm= int((nx/lx_cub)*(x_h_rot + cntr_dis_x)+1)
        j_ibm= int((ny/ly_cub)*(y_h_rot + cntr_dis_y)+1)
        k_ibm= int((nz/lz_cub)*(z_h_rot + cntr_dis_z)+1)

                     if (a.LE.(2.0*thick_a)) then  
                        ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                        sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                        map_ibm(i_ibm,j_ibm,k_ibm)= 1/(1+exp(-sigma_slope))
                     end if


                      !-------interpolation--------------------------
                xh_0 = RR*sin(TT1_0) - a_h_0*sin(TT1_0)*cos(UU_0) - a_h_0*h_h*cos(TT1_0)*sin(UU_0)
                yh_0 = RR*cos(TT1_0) - a_h_0*cos(TT1_0)*cos(UU_0) + a_h_0*h_h*sin(TT1_0)*sin(UU_0)
                zh_0 = h_h*(TT1_0)+a_h_0*RR*sin(UU_0)/(sqrt((RR*RR)+(h_h*h_h)))

                     if ((lag_q>1).and.(i>1).and.(j>1).and.(k>1)) then
                     do ii=1,lag_q
                          interp_dx= xh_0 + real(ii)*(xh-xh_0)/(lag_q+1)
                          interp_dy= yh_0 + real(ii)*(yh-yh_0)/(lag_q+1)
                          interp_dz= zh_0 + real(ii)*(zh-zh_0)/(lag_q+1)
                                  !!!! Rotation
                                 x_rot_0=(-interp_dx*cos(TT1_rot_heli) + interp_dy*sin(TT1_rot_heli))
                                 y_rot_0=interp_dx*sin(TT1_rot_heli) + interp_dy*cos(TT1_rot_heli)
                                 z_rot_0=interp_dz

                                  i_ibm_0= int((nx/lx_cub)*(x_rot_0 + cntr_dis_x)+1)
                                  j_ibm_0= int((ny/ly_cub)*(y_rot_0 + cntr_dis_y)+1)
                                  k_ibm_0= int((nz/lz_cub)*(z_rot_0 + cntr_dis_z)+1)



                     if (a.LE.(2.0*thick_a)) then  
                        ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                        sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                        map_ibm(i_ibm_0,j_ibm_0,k_ibm_0)= 1/(1+exp(-sigma_slope))
                     end if

                     end do
                     end if



         imin = min(imin,i_ibm)
         imax = max(imax,i_ibm)
         jmin = min(jmin,j_ibm)
         jmax = max(jmax,j_ibm)
         kmin = min(kmin,k_ibm)
         kmax = max(kmax,k_ibm)


      end do
    end do
  end do
end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!------------Adding a cilinder----------------!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  !cilinder_head=0                        !Change to 1 to add a cilinder

  if (cilinder_head==1) then
      do k=1,((nz)+1)
         TT1_c=(real(k)-1.0)*Long_c/(nz)
         TT1_c_0=(real(k)-2.0)*Long_c/(nz)

         do j=1,((ny)+1)
           UU_c=(real(j)-1)*theta_u/(ny)
           UU_c_0=(real(j)-2)*theta_u/(ny)

             do i=1,((nx)+1)
               a_c=(real(i)-1.0)*2*thick_a_c/(nx)
               a_c_0=(real(i)-1.0)*2*thick_a_c/(nx)


                    xcilindro = a_c*sin(UU_c)
                    ycilindro = a_c*cos(UU_c)
                    zcilindro = TT1_c

                   !!!! Rotation
                   x_c_rot=(-xcilindro*cos(TT1_rot_heli) + ycilindro*sin(TT1_rot_heli))
                   y_c_rot=xcilindro*sin(TT1_rot_heli) + ycilindro*cos(TT1_rot_heli)
                   z_c_rot=zcilindro;

                     i_ibm= int((nx/lx_cub)*(x_c_rot + cntr_dis_x)+1)
                     j_ibm= int((ny/ly_cub)*(y_c_rot + cntr_dis_y)+1)
                     k_ibm= int((nz/(lz_cub))*(z_c_rot + cntr_dis_z + n_turns*h_h*(2*pi))+1)  !
                     !print*, n_turns*h_h*theta_t

                     if (a_c.LE.(2*thick_a_c)) then
                        ni=ntot*a/(2.0*thick_a_c)  !with this we declare a smoothing thickness and also we improve the smoothing
                        sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                        map_ibm(i_ibm,j_ibm,k_ibm)= 1/(1+exp(-sigma_slope))
                     end if

                   !-------interpolation--------------------------


                    xcilindro_0 = a_c_0*sin(UU_c_0)
                    ycilindro_0 = a_c_0*cos(UU_c_0)
                    zcilindro_0 = TT1_c_0


                     if ((lag_q>1).and.(i>1).and.(j>1).and.(k>1)) then
                     do ii=1,lag_q
                          interp_dx_c= xcilindro_0 + real(ii)*(xcilindro-xcilindro_0)/(lag_q+1)
                          interp_dy_c= ycilindro_0 + real(ii)*(ycilindro-ycilindro_0)/(lag_q+1)
                          interp_dz_c= zcilindro_0 + real(ii)*(zcilindro-zcilindro_0)/(lag_q+1)
                                  !!!! Rotation
                                 x_rot_c_0=(-interp_dx_c*cos(TT1_rot_heli) + interp_dy_c*sin(TT1_rot_heli))
                                 y_rot_c_0=interp_dx_c*sin(TT1_rot_heli) + interp_dy_c*cos(TT1_rot_heli)
                                 z_rot_c_0=interp_dz_c

                                  i_ibm_0= int((nx/lx_cub)*(x_rot_c_0 + cntr_dis_x)+1)
                                  j_ibm_0= int((ny/ly_cub)*(y_rot_c_0 + cntr_dis_y)+1)
                                  k_ibm_0= int((nz/lz_cub)*(z_c_rot + cntr_dis_z + n_turns*h_h*(2*pi))+1)



                                  if (a_c.LE.(2*thick_a_c)) then
                                    ni=ntot*a/(2.0*thick_a_c)  !with this we declare a smoothing thickness and also we improve the smoothing
                                    sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                                    map_ibm(i_ibm_0,j_ibm_0,k_ibm_0)= 1/(1+exp(-sigma_slope))
                                 end if
                     end do
                     end if




             end do
          end do
        end do
     end if

 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!------------Adding a sphere----------------!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 if (esfera_head==1) then

 theta_t_esf=2*pi      
 theta_u_esf=pi      


      do k=1,(nz+1)
         TT1_esf=(real(k)-1.0)*theta_t_esf/(nz)
         TT1_esf_0=(real(k)-2.0)*theta_t_esf/(nz)

         do j=1,(ny+1)
           UU_esf=(real(j)-1)*theta_u_esf/(ny)
           UU_esf_0=(real(j)-2)*theta_u_esf/(ny)

             do i=1,(nx+1)
               a_esf=(real(i)-1.0)*2.0*thick_a_esf/(nx)
               a_esf_0=(real(i)-2.0)*2.0*thick_a_esf/(nx)

                    xesfera = a_esf*cos(TT1_esf)*sin(UU_esf)
                    yesfera = a_esf*sin(TT1_esf)*sin(UU_esf)
                    zesfera = a_esf*cos(UU_esf)
                    

                      !!!! Rotation
                   x_esf_rot=(-xesfera*cos(TT1_rot_heli) + yesfera*sin(TT1_rot_heli))
                   y_esf_rot=xesfera*sin(TT1_rot_heli) + yesfera*cos(TT1_rot_heli)
                   z_esf_rot=zesfera

                     i_ibm= int((nx/lx_cub)*(x_esf_rot + cntr_dis_x)+1)
                     j_ibm= int((ny/ly_cub)*(y_esf_rot + cntr_dis_y)+1)
                     k_ibm= int((nz/lz_cub)*(z_esf_rot + cntr_dis_z + n_turns*h_h*(2*pi)+a_esf)+1)

                                 if (a_esf.LE.(2*thick_a_esf)) then
                                    ni=ntot*a/(2.0*thick_a_esf)  !with this we declare a smoothing thickness and also we improve the smoothing
                                    sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                                    map_ibm(i_ibm,j_ibm,k_ibm)= 1/(1+exp(-sigma_slope))
                                 end if

                      !-------interpolation--------------------------
                       x_esf_0 = a_esf_0*cos(TT1_esf_0)*sin(UU_esf_0)
                       y_esf_0 = a_esf_0*sin(TT1_esf_0)*sin(UU_esf_0)
                       z_esf_0 = a_esf_0*cos(UU_esf_0)


                     if ((lag_q>1).and.(i>1).and.(j>1).and.(k>1)) then
                     do ii=1,lag_q

                          interp_dx_esf= x_esf_0 + real(ii)*(xesfera-x_esf_0)/(lag_q+1)
                          interp_dy_esf= y_esf_0 + real(ii)*(yesfera-y_esf_0)/(lag_q+1)
                          interp_dz_esf= z_esf_0 + real(ii)*(zesfera-z_esf_0)/(lag_q+1)
                                  !!!! Rotation
                                 x_esf_rot_0=(-interp_dx_esf*cos(TT1_rot_heli) + interp_dy_esf*sin(TT1_rot_heli))
                                 y_esf_rot_0=interp_dx_esf*sin(TT1_rot_heli) + interp_dy_esf*cos(TT1_rot_heli)
                                 z_esf_rot_0=interp_dz_esf

                                  i_ibm_0= int((nx/lx_cub)*(x_esf_rot_0 + cntr_dis_x)+1)
                                  j_ibm_0= int((ny/ly_cub)*(y_esf_rot_0 + cntr_dis_y)+1)
                                  k_ibm_0= int((nz/lz_cub)*(z_esf_rot_0 + cntr_dis_z + n_turns*h_h*(2*pi)+a_esf)+1)



                                 if (a_esf.LE.(2*thick_a_esf)) then
                                    ni=ntot*a/(2.0*thick_a_esf)  !with this we declare a smoothing thickness and also we improve the smoothing
                                    sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                                    map_ibm(i_ibm_0,j_ibm_0,k_ibm_0)= 1/(1+exp(-sigma_slope))
                                 end if
                     end do
                     end if
        end do
     end do
   end do
end if


 !!!!!!!!!!!!!!!!!!!!!!!!!---------------irregular mesh-----------------------------!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


 if (Block_helix_irregular==1)  then
  !nx_block= 80    !number of cells inside the 2nd block
  !ny_block= 80    !number of cells inside the 2nd block
  !x_actual_l= 0.0145  !lentgh of the portion
  !y_actual_l= 0.0145  !lentgh of the portion

   if (helix_irregular_y_axis ==1 ) then
   !--------------------helix axis in Y direction---------------------------------------------------------------------
     do lk=1,((ny)+1)
     TT1=(real(lk)-1.0)*theta_t/(ny)
     TT1_0=(real(lk)-2.0)*theta_t/(nz)
       do lj=(1),((nz)+1)
       UU=(real(lj)-1.0)*theta_u/(nz)
       UU_0=(real(lj)-2.0)*theta_u/(nz)

        do li=(1),((nx)+1)
         a_h=(real(li)-1.0)*2*thick_a/(nx)
         a_h_0=(real(li)-2.0)*2*thick_a/(nx)


        xh = RR*sin(TT1) - a_h*sin(TT1)*cos(UU) - a_h*h_h*cos(TT1)*sin(UU)
        yh = h_h*(TT1)+a_h*RR*sin(UU)/(sqrt((RR*RR)+(h_h*h_h)))
        zh = RR*cos(TT1) - a_h*cos(TT1)*cos(UU) + a_h*h_h*sin(TT1)*sin(UU)
        !!!! Rotation
        x_h_rot=(-xh*cos(TT1_rot_heli) + zh*sin(TT1_rot_heli))  !Rotation Matrix application "-" == anticlockwise
        z_h_rot=xh*sin(TT1_rot_heli) + zh*cos(TT1_rot_heli)
        y_h_rot=yh

                !-----------------Morphing into the eulerian mesh------------------------------------------------------------------
        i_ibm= int((nx_block/x_actual_l)*(x_h_rot + cntr_dis_x)+1)
        j_ibm= int((ny_block/y_actual_l)*(y_h_rot + cntr_dis_y)+1)
        k_ibm= int((nz/(lz_cub))*(z_h_rot + cntr_dis_z)+1)

                     if (a.LE.(2.0*thick_a)) then  
                        ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                        sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                        map_ibm(i_ibm,j_ibm,k_ibm)= 1/(1+exp(-sigma_slope))
                     end if

          !-----------------------Interpolation-------------------------------------------------------
                 xh_0 = RR*sin(TT1_0) - a_h_0*sin(TT1_0)*cos(UU_0) - a_h_0*h_h*cos(TT1_0)*sin(UU_0)
                 yh_0 = h_h*(TT1_0)+a_h_0*RR*sin(UU_0)/(sqrt((RR*RR)+(h_h*h_h)))
                 zh_0 = RR*cos(TT1_0) - a_h_0*cos(TT1_0)*cos(UU_0) + a_h_0*h_h*sin(TT1_0)*sin(UU_0)

                     if ((lag_q>1).and.(i>nx_block_pre).and.(j>ny_block_pre).and.(k>1).and.(i<=(nx_block_pre+nx_block)).and.(j<=(ny_block_pre+ny_block))) then
                     do ii=1,lag_q
                          interp_dx= xh_0 + real(ii)*(xh-xh_0)/(lag_q+1)
                          interp_dy= yh_0 + real(ii)*(yh-yh_0)/(lag_q+1)
                          interp_dz= zh_0 + real(ii)*(zh-zh_0)/(lag_q+1)
                                  !!!! Rotation
                                 x_rot_0=(-interp_dx*cos(TT1_rot_heli) + interp_dy*sin(TT1_rot_heli))
                                 y_rot_0=interp_dx*sin(TT1_rot_heli) + interp_dy*cos(TT1_rot_heli)
                                 z_rot_0=interp_dz

                                  i_ibm_0= int((nx_block/x_actual_l)*(x_rot_0 + cntr_dis_x)+1)
                                  j_ibm_0= int((ny_block/y_actual_l)*(y_rot_0 + cntr_dis_y)+1)
                                  k_ibm_0= int((nz/lz_cub)*(z_rot_0 + cntr_dis_z)+1)



                         if (a.LE.(2.0*thick_a)) then  
                            ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                            sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                            map_ibm(i_ibm_0,j_ibm_0,k_ibm_0)= 1/(1+exp(-sigma_slope))
                         end if
                     end do
                     end if







         imin = min(imin,i_ibm)
         imax = max(imax,i_ibm)
         jmin = min(jmin,j_ibm)
         jmax = max(jmax,j_ibm)
         kmin = min(kmin,k_ibm)
         kmax = max(kmax,k_ibm)



       end do
      end do
     end do

    else if (helix_irregular_y_axis ==0 ) then
    !--------------------helix axis in Z direction---------------------------------------------------------------------
         do lk=1,((nz)+1)
          TT1=(real(lk)-1.0)*theta_t/nz
          TT1_0=(real(lk)-2.0)*theta_t/nz
             do lj=(1),((ny)+1)
              UU=(real(lj)-1.0)*theta_u/(ny)
              UU_0=(real(lj)-2.0)*theta_u/(ny)
               do li=(1),((nx)+1)
                a_h=(real(li)-1.0)*2*thick_a/(nx)
                a_h_0=(real(li)-2.0)*2*thick_a/(nx)


        xh = RR*sin(TT1) - a_h*sin(TT1)*cos(UU) - a_h*h_h*cos(TT1)*sin(UU)
        yh = RR*cos(TT1) - a_h*cos(TT1)*cos(UU) + a_h*h_h*sin(TT1)*sin(UU)
        zh = h_h*(TT1)+a_h*RR*sin(UU)/(sqrt((RR*RR)+(h_h*h_h)))

        !!!! Rotation
        x_h_rot=(-xh*cos(TT1_rot_heli) + yh*sin(TT1_rot_heli))  !Rotation Matrix application "-" == anticlockwise
        y_h_rot=xh*sin(TT1_rot_heli) + yh*cos(TT1_rot_heli)
        z_h_rot=zh


        !-----------------Morphing into the eulerian mesh------------------------------------------------------------------
        i_ibm= int((nx_block/x_actual_l)*(x_h_rot + cntr_dis_x)+1)
        j_ibm= int((ny_block/y_actual_l)*(y_h_rot + cntr_dis_y)+1)
        k_ibm= int((nz/(lz_cub))*(z_h_rot + cntr_dis_z)+1)

                     if (a.LE.(2.0*thick_a)) then  
                        ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                        sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                        map_ibm(i_ibm,j_ibm,k_ibm)= 1/(1+exp(-sigma_slope))
                     end if

                            
          !-----------------------Interpolation-------------------------------------------------------
                 xh_0 = RR*sin(TT1_0) - a_h_0*sin(TT1_0)*cos(UU_0) - a_h_0*h_h*cos(TT1_0)*sin(UU_0)
                 yh_0 = h_h*(TT1_0)+a_h_0*RR*sin(UU_0)/(sqrt((RR*RR)+(h_h*h_h)))
                 zh_0 = RR*cos(TT1_0) - a_h_0*cos(TT1_0)*cos(UU_0) + a_h_0*h_h*sin(TT1_0)*sin(UU_0)

                     if ((lag_q>1).and.(i>nx_block_pre).and.(j>ny_block_pre).and.(k>1).and.(i<=(nx_block_pre+nx_block)).and.(j<=(ny_block_pre+ny_block))) then
                     do ii=1,lag_q
                          interp_dx= xh_0 + real(ii)*(xh-xh_0)/(lag_q+1)
                          interp_dy= yh_0 + real(ii)*(yh-yh_0)/(lag_q+1)
                          interp_dz= zh_0 + real(ii)*(zh-zh_0)/(lag_q+1)
                                  !!!! Rotation
                                 x_rot_0=(-interp_dx*cos(TT1_rot_heli) + interp_dy*sin(TT1_rot_heli))
                                 y_rot_0=interp_dx*sin(TT1_rot_heli) + interp_dy*cos(TT1_rot_heli)
                                 z_rot_0=interp_dz

                                  i_ibm_0= int((nx_block/x_actual_l)*(x_rot_0 + cntr_dis_x)+1)
                                  j_ibm_0= int((ny_block/y_actual_l)*(y_rot_0 + cntr_dis_y)+1)
                                  k_ibm_0= int((nz/lz_cub)*(z_rot_0 + cntr_dis_z)+1)


                        if (a.LE.(2.0*thick_a)) then  
                          ni=ntot*a/(2.0*thick_a)  !with this we declare a smoothing thickness and also we improve the smoothing
                          sigma_slope = (smoothslope - ni*2.0*smoothslope/ntot)                               !(a-(1.0-smooth_portion)+smoothslope)-(2*smoothslope*(thick_a)/(nx*smooth_portion))
                          map_ibm(i_ibm_0,j_ibm_0,k_ibm_0)= 1/(1+exp(-sigma_slope))
                        end if
                     end do
                     end if



         imin = min(imin,i_ibm)
         imax = max(imax,i_ibm)
         jmin = min(jmin,j_ibm)
         jmax = max(jmax,j_ibm)
         kmin = min(kmin,k_ibm)
         kmax = max(kmax,k_ibm)

       end do
     end do
    end do
  end if
end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!---------------------------------------------!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  !lx_p(mm) = coor(imax,2)%xp - coor(imin,2)%xp
  !ly_p(mm) = coor(2,jmax)%yp - coor(2,jmin)%yp
  !lz_p(mm) = (kmax-kmin) * dist(2,2)%dzp

  !xi_p(mm) = (coor(imax,2)%xp + coor(imin,2)%xp) / 2
  !yi_p(mm) = (coor(2,jmax)%yp + coor(2,imin)%yp) / 2
  !zi_p(mm) = (kmax+kmin) * dist(2,2)%dzp / 2 + z2


  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 !map_ibm(:,:,:)=0
 !      allocate (map_ibm_tempo(256,256,256))
      ! open(unit=999,file='helix_ibm.dat')

      !  do while (io==0)
      !    read(999,*,iostat=io) i,j,k,tauibm
      !    if (io==0) map_ibm(i+25,j+25,k+50) = tauibm
      !  end do
      !  close(999)


   select case (i_dir)

     ! i_dir = 0 Calculation at the center of the pressure cell
     ! i_dir = 1 Calculation at the center of the velocity cell U
     ! i_dir = 2 Calculation at the center of the velocity cell V
     ! i_dir = 3 Calculation at the center of the velocity cell W

     !-----------------------------------------------
     case(0) ! CALCUL DE ALPHA_P AU POINT DE PRESSION
     !-----------------------------------------------

        do lk=1,nvz_p(mm)
          do lj=1,nvy_p(mm)
            do li=1,nvx_p(mm)
              !print*, nvx_p(mm) , nvy_p(mm) , nvz_p(mm)
              l = 1+(li-1)+(lj-1)*nvx_p(mm)+(lk-1)*nvx_p(mm)*nvy_p(mm)
              !print*, map_ibm(li,lj,lk)
              ! New indice of alpha_ibm vector
              i_box = iv_p(li,mm)
              j_box = jv_p(lj,mm)
              k_box = kv_p(lk,mm)
              !print*, iv_p(li,mm), jv_p(lj,mm), kv_p(lk,mm), nn5_p, nn6_p
              !print*, i_box, j_box, k_box
              alpha_ibm(l) = max(alpha_ibm(l),map_ibm(i_box,j_box,k_box))
              !print*, alpha_ibm(l)
            end do
          end do
        end do
    !-------------------------------------------------
     case(1) ! CALCUL DE ALPHAX_P AU POINT DE VITESSE U
     !-------------------------------------------------
        do lk=1,nvz_p(mm)
          do lj=1,nvy_p(mm)
            do li=1,nvx_p(mm)
             !print*, nvx_p(mm) , nvy_p(mm) , nvz_p(mm)
              l = 1+(li-1)+(lj-1)*nvx_p(mm)+(lk-1)*nvx_p(mm)*nvy_p(mm)

              i_box = iv_p(li,mm)
              j_box = jv_p(lj,mm)
              k_box = kv_p(lk,mm)

              !print*, iv_p(li,mm), jv_p(lj,mm), kv_p(lk,mm), nn5_p, nn6_p
              alpha_ibm(l) = max(alpha_ibm(l),(map_ibm(i_box-1,j_box,k_box)+map_ibm(i_box,j_box,k_box))/2)
               !print*, alpha_ibm(l), (map_ibm(i_box-1,j_box,k_box)+map_ibm(i_box,j_box,k_box))/2
            end do
          end do
        end do

     !-------------------------------------------------
     case(2) ! CALCUL DE ALPHAY_P AU POINT DE VITESSE V
     !-------------------------------------------------
        do lk=1,nvz_p(mm)
          do lj=1,nvy_p(mm)
            do li=1,nvx_p(mm)

              l = 1+(li-1)+(lj-1)*nvx_p(mm)+(lk-1)*nvx_p(mm)*nvy_p(mm)

              i_box = iv_p(li,mm)
              j_box = jv_p(lj,mm)
              k_box = kv_p(lk,mm)

              alpha_ibm(l) = max(alpha_ibm(l),(map_ibm(i_box,j_box-1,k_box)+map_ibm(i_box,j_box,k_box))/2)
              !print*, alpha_ibm(l), (map_ibm(i_box-1,j_box,k_box)+map_ibm(i_box,j_box,k_box))/2
            end do
          end do
        end do

     !-------------------------------------------------
     case(3) ! CALCUL DE ALPHAZ_P AU POINT DE VITESSE W
     !-------------------------------------------------
        do lk=1,nvz_p(mm)
          do lj=1,nvy_p(mm)
            do li=1,nvx_p(mm)

              l = 1+(li-1)+(lj-1)*nvx_p(mm)+(lk-1)*nvx_p(mm)*nvy_p(mm)

              i_box = iv_p(li,mm)
              j_box = jv_p(lj,mm)
              k_box = kv_p(lk,mm)

              alpha_ibm(l) = max(alpha_ibm(l),(map_ibm(i_box,j_box,k_box-1)+map_ibm(i_box,j_box,k_box))/2)
            !print*, map_ibm(i_box,j_box,k_box)
            end do
          end do
        end do



      !-----------
      case default
      !-----------

        print *, 'ATTENTION 0<=i_dir<=3 *** STOP (volfrac_ibm_read) ***'
        stop

      end select

  return
  !IF (ALLOCATED (map_ibm)) DEALLOCATE (map_ibm)
end subroutine volfrac_ibm_helix
