subroutine lagrangian_tracker

use mpi
use commondata
use particle
use sim_par

#define twowayc twowaycflag
#define tracer tracerflag

double precision, dimension(3) :: ftot
integer :: i,j,k,i_loc

! loop over time substeps
do k=1,subiterations
 ! loop over nset
 do j=1,nset
  ! loop over particles
    do i=1+fstartp,nmm+fstartp
       i_loc=i-fstartp     ! local index of particle
#if tracer == 1
    ! time integration ( Explicit Euler ) for tracer
    call lagran4(xp(i,:,j),up(i,:,j))
    xp(i,:,j)=xp(i,:,j)+dt_part*re*up(i,:,j)
    ! tracer velocity is fluid velocity at particle position
    call lagran4(xp(i,:,j),up(i,:,j))
#elif tracer == 0
    ! time integration ( Explicit Euler ) for inertial particles
    xp(i,:,j)=xp(i,:,j)+dt_part*re*up(i,:,j)

    !for multiple sets of particles: when using tracer St=0,tracer velocity is fluid velocity
    !at particle position
    if(abs(stokes(j)).lt.1.0e-15)then
      call lagran4(xp(i,:,j),up(i,:,j))
    else
      up(i,:,j)=up(i,:,j)+dt_part*re*for(i_loc,:,j)
    endif

#endif

    ! check periodicity
    if(xp(i,1,j).lt.0.0d0) then
     xp(i,1,j)=xp(i,1,j)+xl*re
    elseif(xp(i,1,j).gt.xl*re)then
     xp(i,1,j)=xp(i,1,j)-xl*re
    endif
    if(xp(i,2,j).lt.0.0d0) then
     xp(i,2,j)=xp(i,2,j)+yl*re
    elseif(xp(i,2,j).gt.yl*re)then
     xp(i,2,j)=xp(i,2,j)-yl*re
    endif
    ! check rebound (elastic), rebound at particle surface
    if(xp(i,3,j).lt.d_par(j)/2.0d0)then
     xp(i,3,j)=d_par(j)-xp(i,3,j)
     up(i,3,j)=-up(i,3,j)
    elseif(xp(i,3,j).gt.2.0d0*re-d_par(j)/2.0d0)then
     xp(i,3,j)=4.0d0*Re-d_par(j)-xp(i,3,j)
     up(i,3,j)=-up(i,3,j)
    endif
  enddo
 enddo
enddo

call mpi_win_fence(0,window_xp,ierr)
call mpi_win_fence(0,window_up,ierr)


! update forces on particle with new xp and up (for the current time step),only inertila particle
#if tracer == 0
 do j=1,nset
  do i=1+fstartp,nmm+fstartp
    i_loc=i-fstartp
       call calculate_forces(xp(i,:,j),up(i,:,j),i,j,ftot)
       for(i_loc,:,j)=ftot
  enddo
 enddo
#endif


#if twowayc == 1 && tracer == 0
! calculate feedback forces on fluid (two-way coupling only)
fb_x=0.0d0
fb_y=0.0d0
fb_z=0.0d0
do i=0,ntask_sh-1
 if(rank_loc.eq.i)then
  do j=part_index(rank_loc+1,1)+1,part_index(rank_loc+1,1)+part_index(rank_loc+1,2)
   ! add force contribution, to be filled in
   ! fb_x=fb_x+something(j)
   ! fb_y=fb_y+something(j)
   ! fb_z=fb_z+something(j)
  enddo
 endif
 ! synchronize windows
 call mpi_win_fence(0,window_fx,ierr)
 call mpi_win_fence(0,window_fy,ierr)
 call mpi_win_fence(0,window_fz,ierr)
enddo

#endif

return
end subroutine

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine calculate_forces(pos,vel,ipa,case,ftot)

use particle
use sim_par
use phase_field

double precision, dimension(3) :: pos,vel,fgradphi_pp,np
double precision, dimension(3) :: velf,fdrag,fbg,ftot,fsurf
double precision :: Re_p,phi_pp,absfgradphi_pp,mindist
integer :: case,ipa,ipa_loc

#define stokes_drag stokesflag
#define part_grav activategravity
#define part_surface activateSurfaceForce

ftot=0.0d0
fdrag=0.d0
fbg=0.d0
fsurf=0.d0

ipa_loc=ipa-fstartp

 ! for multiple sets of particles: when using tracer St=0, thus ftot=NaN
if(abs(stokes(case)).lt.1.0e-15)then
   ftot=0.d0
   !calculate phi at tracer position
#if part_surface == 1
   call lagran4_phi(pos,phi_pp)
#endif
else ! forces calculation on particle

  ! get fluid velocity at particle position
  call lagran4(pos,velf)

  ! particle Reynolds number
  Re_p=abs(dsqrt((velf(1)-vel(1))**2+(velf(2)-vel(2))**2+(velf(3)-vel(3))**2))*d_par(case)



#if stokes_drag == 1
  ! drag force (Stokes drag)
  fdrag=(velf-vel)/stokes(case)
#elif stokes_drag == 0
  ! drag force (with Schiller-Naumann correction)
  fdrag=(velf-vel)/stokes(case)*(1.0d0+0.15d0*re_p**0.687)
#endif

#if part_grav == 1
   ! buoyancy and gravity, in grav ordering is x,z,y
   fbg(1)=1.0d0/(re*Fr**2)*(dens_part(case)-1.0d0)/dens_part(case)*grav(1)
   fbg(2)=1.0d0/(re*Fr**2)*(dens_part(case)-1.0d0)/dens_part(case)*grav(3)
   fbg(3)=1.0d0/(re*Fr**2)*(dens_part(case)-1.0d0)/dens_part(case)*grav(2)
#endif

#if part_surface == 1
   call lagran4_phi(pos,phi_pp)
   call lagran4_gradphi(pos,fgradphi_pp)

   !calculate Surface Tension force
 if(phi_pp.ge.-phi_crt.and.phi_pp.le.phi_crt) then

   absfgradphi_pp=dsqrt(fgradphi_pp(1)**2+fgradphi_pp(2)**2+fgradphi_pp(3)**2)

   ! signphi_pp=int(phi_pp(ipa,stk)/abs(phi_pp(ipa,stk)))
   np(1)=fgradphi_pp(1)/absfgradphi_pp
   np(2)=fgradphi_pp(2)/absfgradphi_pp
   np(3)=fgradphi_pp(3)/absfgradphi_pp
   mindist=-(dsqrt(2.d0)/2.d0*ch*dlog((1.d0+phi_pp)/(1.d0-phi_pp))*Re)
   ! nptot=dsqrt(npx**2+npy**2+npz**2)

   fsurf(1)=(6.d0*surf_con/dens_part(case))*(Re/We)*(mindist/d_par(case)**3.d0)*np(1)
   fsurf(2)=(6.d0*surf_con/dens_part(case))*(Re/We)*(mindist/d_par(case)**3.d0)*np(2)
   fsurf(3)=(6.d0*surf_con/dens_part(case))*(Re/We)*(mindist/d_par(case)**3.d0)*np(3)
   ! fsurf_tot=dsqrt(fsurf(1)**2+fsurf(2)**2+fsurf(3)**2)
 endif

#endif

  !total force on the particle
  ftot(1)=fdrag(1)+fbg(1)+fsurf(1)
  ftot(2)=fdrag(2)+fbg(2)+fsurf(2)
  ftot(3)=fdrag(3)+fbg(3)+fsurf(3)

endif


partv(ipa_loc,1,case)=phi_pp
partv(ipa_loc,2,case)=fdrag(1)
partv(ipa_loc,3,case)=fdrag(2)
partv(ipa_loc,4,case)=fdrag(3)
partv(ipa_loc,5,case)=fsurf(1)
partv(ipa_loc,6,case)=fsurf(2)
partv(ipa_loc,7,case)=fsurf(3)



return
end
