program FLOW_36

use mpi
use commondata
use mpiIO
use par_size
use grid
use velocity
use sim_par
use phase_field
use stats
use surfactant
use temperature
use particle
use wavenumber
use comm_pattern
use ibm_membrane
use three_phase
#define machine machineflag
#define phiflag phicompflag
#define psiflag psicompflag
#define tempflag tempcompflag
#define stat_dump stats_dump_frequency
#define particles particlecompflag
#define fdump physical_dump_frequency
#define sdump spectral_dump_frequency
#define cpiflag cpicompflag
#define tracer tracerflag
#define part_surface activateSurfaceForce
#define ibmflag ibmcompflag
#define helix_flag ibmhelixflag
#define astroidal_flag astroidalflag
#define elipsoid_flag elipsoidflag
#define helicoid_flag helicoidflag
#define masspoint_flag masspointflag
#define psisol_flag psisolflag
integer :: i,j,k
integer :: dims(2)
integer :: g_size(3),s_size(3)
integer :: fysize(nycpu),fzsize(nzcpu)
integer :: cysize(nzcpu),cxsize(nycpu)
integer :: l_comm, stept_0

double precision :: stime,etime,dtime,mtime,gstime,getime,time
double precision :: dx,dy
double precision :: int_1
!double precision, allocatable, dimension(:,:) :: xh_lag, yh_lag, zh_lag
!double precision, allocatable, dimension(:,:,:) :: massiv_xyz, mass_vel_lag
!!double precision, allocatable, dimension(:,:,:,:) :: mvel
double precision, allocatable, dimension(:,:,:) ::  rest_inv_beam
double precision, allocatable, dimension(:,:,:) :: ang_bend
double precision, allocatable, dimension(:,:) ::lenteja ,rest_length

character(len=5) :: namevar
character(len=50) :: string

logical :: periodic(2), reorder

! start MPI
call mpi_init(ierr)


call split_comm


! check min(nx,ny,nz)>=max(nycpu,nzcpu)
if(min(nx/2,ny).lt.nycpu.or.min(ny,nz).lt.nzcpu) then
  if(rank.eq.0) write(*,*) '#################################'
  if(rank.eq.0) write(*,*) ' STOP, LESS THAN 1 POINT PER CPU'
  if(rank.eq.0) write(*,*) '#################################'
else

  ! read simulation parameters, done in parallel from each task, and print
  ! initial simulation parameters
  call read_input
  ! define array sizes
  call define_sizes

if(rank.lt.flow_comm_lim)then
  ! create cartesian communicator
  dims(1)=nzcpu
  dims(2)=nycpu
  ! for use mpi_f08
  !periodic(1)=1
  !periodic(2)=1
  !reorder=0
  periodic(1)=.true.
  periodic(2)=.true.
  reorder=.false.

  call mpi_cart_create(flow_comm,2,dims,periodic,reorder,cart_comm,ierr)


  ! create derived datatype used in MPI I/O and commit it
  call mpi_cart_sub(cart_comm,[.false.,.true.],l_comm,ierr)
  call mpi_allgather(fpy,1,mpi_integer,fysize,1,mpi_integer,l_comm,ierr)
  call mpi_allgather(spx,1,mpi_integer,cxsize,1,mpi_integer,l_comm,ierr)
  call mpi_comm_free(l_comm,ierr)

  call mpi_cart_sub(cart_comm,[.true.,.false.],l_comm,ierr)
  call mpi_allgather(fpz,1,mpi_integer,fzsize,1,mpi_integer,l_comm,ierr)
  call mpi_allgather(spy,1,mpi_integer,cysize,1,mpi_integer,l_comm,ierr)
  call mpi_comm_free(l_comm,ierr)

  g_size=[nx, nz, ny]
  s_size=[nx, fpz, fpy]
  fstart=[0, 0, 0]
  cstart=[0, 0, 0]
  do i=1,mod(rank,nycpu)
    fstart(3)=fstart(3)+fysize(i)
    cstart(1)=cstart(1)+cxsize(i)
  enddo
  do i=1,floor(real(rank)/real(nycpu))
    fstart(2)=fstart(2)+fzsize(i)
    cstart(3)=cstart(3)+cysize(i)
  enddo
! physical space saving
  call mpi_type_create_subarray(3,g_size,s_size,fstart,mpi_order_fortran, &
   &     mpi_double_precision,ftype,ierr)

  call mpi_type_commit(ftype,ierr)

! spectral space saving (coarse grid)
  call shrink_mapping


! define dual grid sizes
  call define_dual_grid(cxsize,cysize)

! spectral space saving (fine grid)
  call shrink_mapping_fg

! define dual grid communication pattern
  call define_address
endif

  ! create fftw plans
  if(rank.eq.0) then
    write(*,*) 'Starting FFTW plan creation ...'
    write(*,*) 'This may take a while'
  endif
  stime=mpi_wtime()
  call create_plan
  call create_plan_fg
  etime=mpi_wtime()
  dtime=etime-stime
  call mpi_reduce(dtime,mtime,1,mpi_double,mpi_sum,0,mpi_comm_world,ierr)
  mtime=mtime/dble(ntask)
  string='Time elapsed for FFTW plan creation='
  if(rank.eq.0) call write_time(string,mtime)


  !define grid
  if(nx.gt.1.and.ny.gt.1)then
    dx=xl/dble(nx-1)
    dy=yl/dble(ny-1)
  else
    write(*,*) 'error, not enough points'
    stop
  endif

  x(1)=0.0d0
  y(1)=0.0d0
  do i=2,nx
    x(i)=x(i-1)+dx
  enddo
  do j=2,ny
    y(j)=y(j-1)+dy
  enddo

  do k = 1, nz
    z(k)=dcos(((k-1)*pi)/(nz-1))
  enddo

  ! save grid arrays
  if(rank.eq.0) call dump_grid


! executed only by flow_comm
if(rank.lt.flow_comm_lim)then
  ! calculate wavenumbers
  call wave_numbers
  ! initialize flow field according to initial condition provided
  ! allocate velocity matrices in physical and modal space

  call initialize

#if phiflag == 1
  call initialize_phi
#if psiflag == 1
  call initialize_psi
#endif
#endif

#if tempflag == 1
  call initialize_temp
#endif

#if stat_dump > 0
    call initialize_stats
#endif


!-------------------------------------------------------------

!-------------kevin-################--------------------------
#if ibmflag == 1
call ibm_read_input
!Distribution of particles respect to the processors
!write(*,*) ntask_gl
if(mod(n_budies,ntask_gl).ne.0) then
  if (rank.eq.(ntask_gl-1)) then
    n_budies_ini_rank=rank*floor(real(n_budies/ntask_gl)) +1
    n_budies_fin_rank=(rank+1)*floor(real(n_budies/ntask_gl))+mod(n_budies,ntask_gl)
  else
    n_budies_ini_rank=rank*floor(real(n_budies/ntask_gl)) +1
    n_budies_fin_rank=(rank+1)*floor(real(n_budies/ntask_gl))
  endif
elseif(mod(n_budies,ntask_gl).eq.0) then
  n_budies_ini_rank=rank*floor(real(n_budies/ntask_gl)) +1
  n_budies_fin_rank=(rank+1)*floor(real(n_budies/ntask_gl))
!#### For fewer particles than ranks #############
elseif(n_budies.lt.ntask_gl) then
  n_budies_ini_rank=(rank+1)
  n_budies_fin_rank=(rank+1)
endif
!Checking if works!write(*,*) 'number of budies per task', n_budies/ntask_gl
!Checking if works!write(*,*) 'rank, nbudini, n_budfin', rank, n_budies_ini_rank,n_budies_fin_rank

!%%%%%%%%%%%%%%%%%%%%%%%%%
allocate (xh_lagrang(n_budies,nz_h))
allocate (yh_lagrang(n_budies,nz_h))
allocate (zh_lagrang(n_budies,nz_h))
allocate (rest_length(n_budies,nz_h))
allocate (lenteja(n_budies,nz_h))
allocate (ang_bend(n_budies,nz_h,3))
allocate (rest_inv_beam(n_budies,nz_h,3))
allocate (old_forza(nx,ny,nz))
xh_lagrang=0.0d0
yh_lagrang=0.0d0
zh_lagrang=0.0d0

rest_length=0.0d0
lenteja=0.0d0
old_forza(:,:,:)=0.0d0
#if astroidal_flag == 1
do i_bud=1,n_budies
  if(few_bod.lt.ntask_gl) then
    if(i_bud.le.few_bod) then

 call ini_bended(xh_lagrang(i_bud,nz_h), yh_lagrang(i_bud,nz_h), zh_lagrang(i_bud,nz_h),&
 rest_length(i_bud,nz_h),ang_bend(i_bud,nz_h,3),lenteja(i_bud,nz_h),&
 rest_inv_beam(i_bud,nz_h,3),i_bud)
    endif
  else
  call ini_bended(xh_lagrang(i_bud,nz_h), yh_lagrang(i_bud,nz_h), zh_lagrang(i_bud,nz_h),&
  rest_length(i_bud,nz_h),ang_bend(i_bud,nz_h,3),lenteja(i_bud,nz_h),&
  rest_inv_beam(i_bud,nz_h,3),i_bud)
  endif
enddo

! #if psisol_flag == 1
! call initialize_psisol
! do i_bud=1,n_budies
!   if(few_bod.lt.ntask_gl) then
!     if(i_bud.le.few_bod) then
!     call ini_pbodies(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1),i_bud)
!     endif
!   else
!     call ini_pbodies(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1),i_bud)
!   endif
! enddo
!
! !######## writing p_phisol ###########
! stept_0=0
! call genpsi(stept_0)
! !######## writing p_phisol ###########
! #endif

#endif

#if elipsoid_flag == 1
call ini_elipsoid(xh_lag, yh_lag, zh_lag)
#endif

#if helix_flag == 1
call ini_helix(xh_lag, yh_lag, zh_lag)
#endif

#if helicoid_flag == 1
call ini_helicoid(xh_lag, yh_lag, zh_lag)
#endif

#if masspoint_flag == 1
allocate (massiv_xyz_all(n_budies,nz_h,3))
allocate (mass_vel_lag_all(n_budies,nz_h,3))
allocate(massiv_xyz_half_all(n_budies,nz_h,3))
allocate(mass_vel_lag_half_all(n_budies,nz_h,3))
allocate(mass_vel_lag_prev_all(n_budies,nz_h,3))


!initilazed at the same position as the lagrangian masless

mass_vel_lag_all=0.0d0

!mvel=0.0d0
! mass_vel_lag=0.0d0
mass_vel_lag_half_all=0.0d0
mass_vel_lag_prev_all=0.0d0
mass_vel_lag_all=0.0d0

if(resume_flag.eq.0) then
mass_vel_lag_all=0.0d0

!mvel=0.0d0
! mass_vel_lag=0.0d0
mass_vel_lag_half_all=0.0d0
mass_vel_lag_prev_all=0.0d0
mass_vel_lag_all=0.0d0
elseif(resume_flag.eq.1) then
do i_bud=1,n_budies
  if(few_bod.lt.ntask_gl) then
    if(i_bud.le.few_bod) then
      call resume_lvsol(i_bud)
      call resume_lpsol(i_bud)

    endif
  else
  call resume_lvsol(i_bud)
  call resume_lpsol(i_bud)

  endif
enddo
endif

massiv_xyz_all(:,:,1)=xh_lagrang(:,:)
massiv_xyz_all(:,:,2)=yh_lagrang(:,:)
massiv_xyz_all(:,:,3)=zh_lagrang(:,:)
massiv_xyz_half_all=massiv_xyz_all
#endif

#if psisol_flag == 1
call initialize_psisol
do i_bud=1,n_budies
  if(few_bod.lt.ntask_gl) then
    if(i_bud.le.few_bod) then
    call ini_pbodies(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1),i_bud)
    endif
  else
    call ini_pbodies(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1),i_bud)
  endif
enddo
!---------------------------------
!temporal cleaning
#if phiflag == 1
!#################################
 call spectral_to_phys(phic,phi,0)
 call phi_cleanovershot
 call phys_to_spectral(phi,phic,0)
!###################################

#endif
!---------------------------------
!######## writing p_phisol ###########
stept_0=0
call genpsi(stept_0)
!######## writing p_phisol ###########
#endif


#if psisol_flag == 1
#if phiflag == 1

if(inipsisol_flag.eq.1) then
call spectral_to_phys(phic,phi,0)
! intializing single well in phase field
do i_bud=1,n_budies
  if(few_bod.lt.ntask_gl) then
    if(i_bud.le.few_bod) then
    call phi_ibm_ini(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1))
    endif
  else
    call phi_ibm_ini(xh_lagrang(i_bud,1),yh_lagrang(i_bud,1), zh_lagrang(i_bud,1))
  endif
enddo
!--------------------------------------
call phys_to_spectral(phi,phic,0)
endif
#endif

! #if psiflag == 7
! call psi_ibm_ini(xh_lag,yh_lag,zh_lag)
! call phys_to_spectral(psi,psic,0)
! call coarse2fine(psic,psic_fg)
! call spectral_to_phys_fg(psic_fg,psi_fg,0)
! #endif
#endif

#endif

! stop
!---------------################------------------------------
  ! save initial fields
  if(restart.eq.0)then
   call save_flow_comm(nstart)
  endif

#if phiflag == 1
  call integral_phi(int_1)
#endif

 if(restart.eq.0)then
  if(rank.eq.0) call initialize_check(int_1)
 endif
! end of flow_comm only
endif

#if particles == 1
  ! executed only by part_comm, allocates variables
  if(rank.ge.leader) call allocate_particle
  call create_communication_pattern
  call initialize_particle
  if(restart.eq.0)then
  ! save particle data (part_comm)
  if(rank.ge.leader)then
   namevar='pos'
   call write_output_part(xp,nstart,namevar)
   namevar='vel'
   call write_output_part(up,nstart,namevar)
! write fluid velocity at particle position
#if tracer == 0
   call write_output_partf(nstart)
#if part_surface == 1
   namevar='partv'
   call write_partvalue(partv,nstart,namevar)
#endif
#endif
  endif
#if tempflag == 1
    ! get temperature at particle position (done here, after defining all auxiliary arrays)
    call get_temperature
    if(rank.ge.leader) call write_output_partT(nstart)
#endif
#endif

endif

  gstime=mpi_wtime()

  ! loop over time
  time=0.0d0
  string='time elapsed per timestep'

  do i=nstart+1,nend

    stime=mpi_wtime()

    time=time+dt

    ! write to screen simulation status
    if(rank.eq.0)then
      write(*,*) '-----------------------------------------------------------------------'
      write(*,'(1x,a,i8,a,i8,2(8x,a,es9.3))') 'step',i,' of ',nend,'t-=',time,'t+=',time*Re
    endif

    call solver(i, rest_length, ang_bend, lenteja, rest_inv_beam)

    ! save variables and stats (flow_comm)
    if(rank.lt.flow_comm_lim)then
#if stat_dump > 0
     if(mod(i,stat_dump).eq.0.and.i.ge.stat_start.and.i.ne.nstart) then
      if(rank.eq.0) write(*,*) 'writing statistics to file'
      call statistics
     endif
#endif
     call save_flow_comm(i)
    endif

#if particles == 1
    if(((mod(i,ndump).eq.0.and.ndump.gt.0).or. &
     &      (mod(i,sdump).eq.0.and.sdump.gt.0).or. &
     &      (mod(i,part_dump).eq.0)).and.(i.ne.nstart))then
#if tempflag == 1
     ! get temperature at particle position
     ! done here if particle are temperature tracers (do not fetch temperature at each time step)
     call get_temperature
#endif
     ! save particle data (part_comm)
     if(rank.ge.leader)then
      if(rank.eq.leader) write(*,*) 'saving particle solution'
      namevar='pos'
      call write_output_part(xp,i,namevar)
      namevar='vel'
      call write_output_part(up,i,namevar)

      ! write fluid velocity at particle position
#if tracer == 0
      call write_output_partf(i)
#if part_surface == 1
      namevar='partv'
      call write_partvalue(partv,i,namevar)
#endif
#endif
      ! write fluid temperature at particle position
#if tempflag == 1
      call write_output_partT(i)
#endif
     endif
    endif
#endif

#if phiflag == 1
    if(rank.lt.flow_comm_lim) call integral_phi(int_1)
#endif

    if(rank.eq.0) call sim_check(i,int_1)

! only for CPI
#if cpiflag == 1
    if(rank.lt.flow_comm_lim) call mpi_bcast(gradpx,1,mpi_double_precision,0,flow_comm,ierr)
#endif


    ! the MPI all reduce is too costly, so it has been replaced by mean time rank 0
    ! write to screen time elapsed at each time step
    etime=mpi_wtime()
    dtime=etime-stime
!    call mpi_reduce(dtime,mtime,1,mpi_double,mpi_sum,0,mpi_comm_world,ierr)
!    mtime=mtime/dble(ntask)
!    if(rank.eq.0)then
!      call write_time(string,mtime)
!    endif
     if(rank.eq.0) call write_time(string,dtime)


    ! write temporary output files from which the simulation can be restarted if it crashes or stops
    if(mod(i,dump_failure).eq.0)then
      call write_failure(i)
    endif

  enddo
#if ibmflag == 1
deallocate (xh_lagrang)
deallocate (yh_lagrang)
deallocate (zh_lagrang)
deallocate (rest_length)
deallocate (lenteja)
deallocate (ang_bend)
deallocate (rest_inv_beam)

#if masspoint_flag == 1
deallocate (massiv_xyz_all)
deallocate (mass_vel_lag_all)
deallocate(massiv_xyz_half_all)
deallocate(mass_vel_lag_half_all)
deallocate(mass_vel_lag_prev_all)
#endif

#endif
  if(rank.eq.0)then
    write(*,*) '-----------------------------------------------------------------------'
    write(*,*) 'End of simulation'
  endif

  ! write final fields if not already written
  if(rank.lt.flow_comm_lim)then
#if stat_dump > 0
   if(mod(nend,stat_dump).ne.0) then
     call statistics
   endif
   call del_old_stats
#endif
   call save_flow_comm_final(nend)
  endif

  ! save final particle data if not already written
#if particles == 1
#if tempflag == 1
    ! get temperature at particle position
    call get_temperature
#endif
  if((rank.ge.leader).and. &
 &     ((mod(nend,ndump).ne.0.and.ndump.gt.0).or. &
 &      (mod(nend,sdump).ne.0.and.sdump.gt.0).or. &
 &      (mod(nend,part_dump).ne.0)))then
     namevar='pos'
     call write_output_part(xp,nend,namevar)
     namevar='vel'
     call write_output_part(up,nend,namevar)
#if tracer == 0
     call write_output_partf(nend)
#if part_surface == 1
    namevar='partv'
    call write_partvalue(partv,nend,namevar)
#endif
#endif
#if tempflag == 1
     call write_output_partT(nend)
#endif
    endif
#endif

  ! output to screen total time elapsed
  getime=mpi_wtime()
  dtime=getime-gstime

  call mpi_reduce(dtime,mtime,1,mpi_double,mpi_sum,0,mpi_comm_world,ierr)
  mtime=mtime/dble(ntask)
  string='total time elapsed for time advancement='
  if(rank.eq.0) call write_time(string,mtime)


  if(rank.lt.flow_comm_lim)then
   ! deallocate all allocatable arrays
   call destroy
#if phiflag == 1
   call destroy_phi
#if psiflag == 1
   call destroy_psi
#if psisol_flag == 1
   call destroy_psisol
#endif
#endif
#endif
#if tempflag == 1
   call destroy_theta
#endif
  ! destroy MPI I/O derived datatype
   call mpi_type_free(ftype,ierr)
   call mpi_type_free(ftype_fg,ierr)
   if(sp_save_comm.ne.MPI_comm_null) then
     call mpi_type_free(stype,ierr)
     call mpi_comm_free(sp_save_comm,ierr)
   endif
   if(sp_save_comm_fg.ne.MPI_comm_null) then
     call mpi_type_free(stype_fg,ierr)
     call mpi_comm_free(sp_save_comm_fg,ierr)
   endif
   ! destroy cartesian communicator
   call mpi_comm_free(cart_comm,ierr)
   deallocate(kxpsi)
   deallocate(kypsi)
   deallocate(k2psi)
  endif


  ! destroy fftw plans
  call destroy_plan

  ! free all group communicators
  if(rank.lt.flow_comm_lim) call mpi_comm_free(flow_comm,ierr)
  deallocate(stokes,dens_part,d_par)
#if particles == 1
  if(rank.le.flow_comm_lim) call mpi_comm_free(comm_comm,ierr)
  if(rank.ge.leader)then
   call mpi_comm_free(part_comm,ierr)
   call mpi_win_fence(0,window_u,ierr)
   call mpi_win_fence(0,window_v,ierr)
   call mpi_win_fence(0,window_w,ierr)
   call mpi_win_fence(0,window_fx,ierr)
   call mpi_win_fence(0,window_fy,ierr)
   call mpi_win_fence(0,window_fz,ierr)
   call mpi_win_free(window_u,ierr)
   call mpi_win_free(window_v,ierr)
   call mpi_win_free(window_w,ierr)
   call mpi_win_free(window_fx,ierr)
   call mpi_win_free(window_fy,ierr)
   call mpi_win_free(window_fz,ierr)
   call mpi_type_free(part_save,ierr)
   call mpi_type_free(part_save_scalar,ierr)
   call mpi_type_free(partvartype,ierr)
   deallocate(part_index)
   deallocate(saved_size,address_start)
#if tracer == 0
   deallocate(for)
   deallocate(partv)
#endif

  endif
  ! call free(xp)
  ! call free(up)
  ! call free(uf)
  ! call free(vf)
  ! call free(wf)
  ! call free(fb_x)
  ! call free(fb_y)
  ! call free(fb_z)
#endif

!endif

call mpi_finalize(ierr)


end program FLOW_36
