module commondata
 implicit none

 !integer, parameter ::
 integer, parameter :: nx=nnx, ny=nny, nz=nnz
 integer, parameter :: nycpu=nnycpu, nzcpu=nnzcpu
 integer :: rank,ntask,ntask_gl,ntask_sh,rank_loc
 integer :: nodes,leader,flow_comm_lim
! optional parameter for MPI subroutine with USE MPI_F08, included so that the code is retrocompatible with USE MPI
 integer :: ierr
! for use mpi_f08
!type(mpi_comm) :: cart_comm
 integer :: cart_comm,flow_comm,part_comm,comm_comm, cho_sub_t
 double precision :: xl,yl
 character(len=50) :: folder='./results'
end module commondata



module dual_grid
 use commondata
 integer, parameter :: exp_x=expansionx, exp_y=expansiony, exp_z=expansionz
 integer :: npsix,npsiy,npsiz
 integer :: cg_size(nycpu,nzcpu,5),fg_size(nycpu,nzcpu,5)
 integer :: spxpsi,spypsi,fpypsi,fpzpsi
 integer :: cstartpsi(3),fstartpsi(3)
 integer :: c2fadd(nycpu*nzcpu,2),f2cadd(nycpu*nzcpu,2)
end module dual_grid



module grid
 use commondata
 double precision :: x(nx),y(ny),z(nz)
end module grid



module velocity
 double precision, allocatable :: u(:,:,:), v(:,:,:), w(:,:,:)
 double precision, allocatable :: uc(:,:,:,:), vc(:,:,:,:), wc(:,:,:,:)
 double precision, allocatable :: uc_ene(:,:,:,:), vc_ene(:,:,:,:), wc_ene(:,:,:,:)
 double precision, allocatable, dimension(:,:,:) :: u_fg, v_fg, w_fg
 double precision, allocatable, dimension(:,:,:,:) :: uc_fg, vc_fg, wc_fg
 double precision, allocatable :: wa2(:,:,:), wa3(:,:,:)
 double precision, allocatable, dimension(:,:,:,:) :: sgradpx,sgradpy
 double precision, allocatable, dimension(:,:,:) :: forx,fory,forz
end module velocity



module wavenumber
 use commondata
 double precision :: kx(nx/2+1),ky(ny),k2(nx/2+1,ny), k4(nx/2+1,ny)
 double precision, allocatable :: kxpsi(:),kypsi(:),k2psi(:,:)
end module wavenumber



module sim_par
 double precision, parameter :: pi=3.14159265358979
 double precision :: Re, dt, gradpx, gradpy, Co, repow
 double precision :: gamma
! b.c. on w and omega for Helmholtz problem
 double precision, dimension(2) :: p_u,q_u,r_u,p_v,q_v,r_v,p_w,q_w,r_w,p_o,q_o,r_o,zp
 integer :: bc_up, bc_low
 integer :: cpi_flag
 integer :: restart, nt_restart, in_cond, dump_failure
 integer :: nstart, nend, ndump,sdump
end module sim_par



module phase_field
 integer :: phi_flag,phicor_flag,in_cond_phi,b_type,phiflux_flag,body_flag,ele_flag, non_newtonian
 double precision :: rhor,visr,We,Ch,Pe,Fr,grav(3),s_coeff,lamphi,body_c,body_d(3),stuart
 double precision :: muinfmuzero, exp_non_new
 double precision, allocatable :: phi(:,:,:), phic(:,:,:,:),phi_f(:,:,:)
 double precision, allocatable :: phi_fg(:,:,:), phic_fg(:,:,:,:)
 double precision,allocatable :: fgradphix_f(:,:,:),fgradphiy_f(:,:,:),fgradphiz_f(:,:,:)
end module phase_field

!########### Three phase field ##################################
module three_phase
integer :: psisol_flag  !switch on-off
integer, parameter:: inipsisol_flag = 0  !switch on-off
integer, parameter:: resume_flag = 0, resumetimes=15000  !resume lag position and velocities from .dat in reuslts
double precision, allocatable :: psi_sol(:,:,:), psi_solc(:,:,:,:)
double precision, allocatable :: p_psisol(:,:,:)
double precision, allocatable :: p_psisolc(:,:,:,:)
double precision, allocatable :: p_alfa_psi_sol(:,:,:,:) !paralelized
! double precision, allocatable :: ch_psisolc(:,:,:,:)   !spectral
double precision, allocatable, dimension(:,:,:,:) :: frz_ibm_rank, frz_ibm
!feedback into Navier stokes
double precision, allocatable :: aeu1f(:,:,:), aeu2f(:,:,:), aeu3f(:,:,:)
double precision, allocatable, dimension(:,:,:,:) :: aeu1,aeu2,aeu3,aeu4,aeu5,aeu6,aeu7
double precision, allocatable :: fnsx_psisol(:,:,:),fnsy_psisol(:,:,:),fnsz_psisol(:,:,:)   !spectral
double precision, allocatable :: fnsx_psisolc(:,:,:,:),fnsy_psisolc(:,:,:,:),fnsz_psisolc(:,:,:,:)   !spectral
double precision, allocatable, dimension(:,:,:,:) :: spsi_solo
real, parameter :: eta_fp=0.02, eta_dp=0.02, wet_par=0.03! -0.27 0.24 !affitinity
real, parameter :: s_thikness=0.0 ,carrier_solid=-1.0
real, parameter :: coef_ternary=800.0
double precision :: cut_flux
end module three_phase
!################################################################

module surfactant
! mean surfactant concentration, initial condition parameters, ...
 integer :: psi_flag, in_cond_psi
 double precision :: Ex, Pe_psi, P_i, El
 double precision, allocatable :: psi(:,:,:),psic(:,:,:,:)
 double precision, allocatable :: psi_fg(:,:,:),psic_fg(:,:,:,:)
end module surfactant



module temperature
 integer :: theta_flag,in_cond_theta
 double precision :: Ra,Pr
 double precision, dimension(2) :: p_theta,q_theta,r_theta
 double precision, allocatable :: theta(:,:,:),thetac(:,:,:,:)
end module temperature



module particle
 integer :: part_flag,part_number,in_cond_part_pos,in_cond_part_vel,part_dump,nset,nmm,fstartp
 integer :: subiterations,part_capillary_flag,in_cond_part_var
 integer, allocatable, dimension(:,:) :: part_index
 double precision :: dt_part,surf_con,phi_crt
 double precision, allocatable ,dimension(:,:,:):: for,partv
 double precision, allocatable, dimension(:) :: stokes,dens_part,d_par
 double precision, pointer, dimension(:,:,:) :: xp,up
 double precision, pointer, dimension(:,:,:) :: uf,vf,wf,Tf,fb_x,fb_y,fb_z,phif
 double precision, pointer, dimension(:,:,:) :: gradphix_phys,gradphiy_phys,gradphiz_phys
 ! mpi shared memory synchronization windows
 integer :: window_u,window_v,window_w,window_T,window_fx,window_fy,window_fz,window_phi
 integer :: window_xp,window_up,window_gradphix,window_gradphiy,window_gradphiz
end module particle



module velocity_old
 ! store velocities at t=n-1, needed for non-linear part of time derivatives for non-matched densities case
 double precision, allocatable :: ucp(:,:,:,:), vcp(:,:,:,:), wcp(:,:,:,:)
end module velocity_old



module mpiIO
 integer :: ftype,stype
 integer :: ftype_fg,stype_fg
 integer :: sp_save_comm,sp_save_comm_fg
 integer :: part_save,part_save_scalar,partvartype
end module mpiIo



module par_size
! size in physical space: nx,npz,npy
 integer :: fpy,fpz
! size in spectral space:
 integer :: spx,spy
 integer :: cstart(3),fstart(3)
end module par_size



module fftw3
 use, intrinsic :: iso_c_binding
 include 'fftw3.f03'
 type(c_ptr) :: plan_x_fwd,plan_y_fwd,plan_z_fwd
 type(c_ptr) :: plan_x_bwd,plan_y_bwd,plan_z_bwd
 type(c_ptr) :: plan_x_fwd_fg,plan_y_fwd_fg,plan_z_fwd_fg
 type(c_ptr) :: plan_x_bwd_fg,plan_y_bwd_fg,plan_z_bwd_fg
end module fftw3



module sterms
 double precision, allocatable, dimension(:,:,:,:) :: s1_o,s2_o,s3_o, sphi_o, spsi_o, stheta_o
end module sterms



module stats
 integer :: flowiter,stat_dump,stat_start
 integer :: plane_comm,col_comm
end module stats



module shrink_grid
 integer :: sx1,sx2,sy1,sy2,sy3,sy4
 integer :: sx1_fg,sx2_fg,sy1_fg,sy2_fg,sy3_fg,sy4_fg
 integer, dimension(3) :: dimc,dimc_fg
 integer :: up,up_fg
end module shrink_grid



module comm_pattern
 integer, dimension(4) :: chunk_size
 integer, allocatable, dimension(:,:) ::saved_size,address_start
end module comm_pattern




module ibm_membrane
!Domain - particle partitioning
integer :: n_budies_ini_rank, n_budies_fin_rank, i_bud
integer :: i_nbudies, i_nz
integer, parameter :: n_budies=numb_bod, few_bod=fewer_nbod
double precision, allocatable, dimension(:,:,:) :: old_forza
!initial position
double precision, parameter :: lx_pos=0.5d0,ly_pos=0.5d0,lz_pos=0.0d0
double precision, allocatable, dimension(:,:) :: xh_lagrang, yh_lagrang, zh_lagrang
double precision, allocatable, dimension(:,:,:) :: massiv_xyz_all, mass_vel_lag_all
double precision, allocatable, dimension(:,:,:) :: massiv_xyz_half_all, mass_vel_lag_half_all
double precision, allocatable, dimension(:,:,:) :: mass_vel_lag_prev_all
!delta indices
double precision :: lx_cub , ly_cub, lz_cub
integer :: x_eul, y_eul, z_eul
!support initialization
integer, parameter :: nx_supp=10,ny_supp=50,nz_supp=50
!support force spreading
integer, parameter :: n_supp=50
double precision, parameter :: ptn=0.0
double precision :: p_new, pnew_0, p_new_01
!solid interfacial thickness
!double precision, parameter :: eta_sol=0.01d0 !since its sigmaid, eta_sol=0.5Ch
double precision :: sigma_slope
!Phobicity
double precision, parameter :: polar_phi=2.0 !range -1 to 1
!radio sphere
double precision, parameter :: rad_esf=0.125d0 !dimensionless
double precision, parameter :: h_dim= 0.02  !meters
!massive
double precision, parameter :: mass=1.979e-3! For 2D mass
double precision, parameter :: rho_fld= 1.2d0
double precision, parameter :: rho_part=1920.0!!1920.0
double precision, parameter :: dyn_visc=1.8e-5 ! for the inertial part
double precision, parameter :: tau_fld=0.75 ! 0.5relaxation time
!2D simulation tresd_sim=0
integer, parameter :: tresd_sim=0
! integer :: tresd_supp
double precision :: vel_shear ! shear velocity
!Force solid body
double precision, allocatable, dimension(:,:) :: solid_force, solid_force_prev
!size of interfacial
double precision, parameter ::  eta_sol=0.01d0
!froude_ibm
double precision, parameter :: froude_ibm=0.96 !0.0274 !0.21
double precision, allocatable, dimension(:,:) :: vel_interp
double precision, allocatable, dimension(:) :: immers, cline_adv
integer, parameter :: flag_sdvis=1
double precision,parameter ::soldynvis=2000.0d0
double precision :: viscurr(3)
!collisions solid solid
integer, parameter :: part_part_coll=0
double precision, allocatable ::fcolssbud(:,:)
!plot parameters figures
double precision, allocatable, dimension(:) :: relfluidrho,addedmass,&
dragforce, buoyforce
double precision, allocatable, dimension(:) :: relfluidrhobud,addedmassbud,&
dragforcebud, buoyforcebud
!astroidal
double precision :: heigth_a

!elipsoid
double precision :: a_eli, b_eli, c_eli

!helix
double precision :: a_h, R_h , h_h , axi_lngth
double precision :: T_h, U_h, T_h0, U_h0
integer,dimension(1) :: k_ibm, j_ibm, i_ibm
integer :: n_turns, tam

!helicoid
double precision :: a_helic, b_helic, c_helic

double precision :: xh, yh, zh
double precision :: xh_0, yh_0, zh_0
double precision :: xh_01, yh_01, zh_01
double precision :: theta_t, theta_u, thick_a
double precision :: ni, ntot
integer :: grid_param(7)
integer, parameter :: nx_h=celgx, ny_h=celgy, nz_h=celgz

end module ibm_membrane
