subroutine sterm_ch(sphi)

use commondata
use par_size
use velocity
use phase_field
use wavenumber
use grid
use three_phase
double precision :: sphi(spx,nz,spy,2),epsnum,mask
double precision :: modnabphi,normflux,funflux(nz)
double precision, allocatable, dimension(:,:,:,:) :: a1,a2,a3
double precision, allocatable, dimension(:,:,:) :: a1f,convf
double precision, allocatable :: uc_les(:,:,:,:), vc_les(:,:,:,:)
double precision, allocatable :: u_les(:,:,:), v_les(:,:,:)
!USED ONLY WHEN THE CORRECTION IS ENABLED
double precision, allocatable, dimension(:,:,:) :: phinx,phiny,phinz
double precision, allocatable, dimension(:,:,:) :: a4f,a5f,a6f,nabcf
double precision, allocatable, dimension(:,:,:) :: tenif,tenjf,tenkf
double precision, allocatable, dimension(:,:,:,:) :: nabc,a4,a5,a6
! To control wetability
double precision, allocatable, dimension(:,:,:) :: psisolnx,psisolny,psisolnz
double precision, allocatable, dimension(:,:,:,:) :: gradmux,gradmuy,gradmuz
double precision :: modnabpsisol

integer :: i,j,k

#define phicorflag phicorcompflag

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 10
! STANDARD MODEL

! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

sphi=-(1.0d0+s_coeff)/pe*sphi

!Calculate convective term
allocate(a1f(nx,fpz,fpy))

! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo
call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo
call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

call phys_to_spectral(convf,a1,1)

deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1

!Calculate nabla^2 \phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3,a1,a1f)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 1
! PROFILE-CORRECTED
! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi

!Calculate convective term
allocate(a1f(nx,fpz,fpy))

! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

allocate(phinz(nx,fpz,fpy))
phinz=a1f

call phys_to_spectral(convf,a1,1)

deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1


!CALCULATE THE LAMBDA TERM (EXPLICIT)
a1f=1.0d0-phi**(2.0d0)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      phinx(i,k,j)=phinx(i,k,j)/modnabphi
      phiny(i,k,j)=phiny(i,k,j)/modnabphi
      phinz(i,k,j)=phinz(i,k,j)/modnabphi
    enddo
  enddo
enddo

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))


do j=1,fpy
  do k=1,fpz
    do i=1,nx
      a4f(i,k,j)=a1f(i,k,j)*phinx(i,k,j)
      a5f(i,k,j)=a1f(i,k,j)*phiny(i,k,j)
      a6f(i,k,j)=a1f(i,k,j)*phinz(i,k,j)
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))
allocate(a5(spx,nz,spy,2))
allocate(a6(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a5(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a5(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a6)

!SUM EVEYTHING TO SPHI
sphi=sphi-(lamphi/(sqrt(2.0d0)*ch*pe))*(a4+a5+a6)

deallocate(a4,a5,a6)

!Calculate nabla^2 \phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3,a1,a1f)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 2
! Flux-Corrected

allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

allocate(nabc(spx,nz,spy,2))
nabc=ch*ch*sphi
sphi=-(s_coeff-lamphi)/pe*sphi

!Calculate convective term
allocate(a1f(nx,fpz,fpy))

! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)
allocate(phinx(nx,fpz,fpy))
phinx=a1f

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)
allocate(phiny(nx,fpz,fpy))
phiny=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)
allocate(phinz(nx,fpz,fpy))
phinz=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

call phys_to_spectral(convf,a1,1)
deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1

! computing 1/Pe nabla^2 phi^3
a1f=phi**(3d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3)

!COMPUTING THE DIVERGENCE TERM..
nabc=-a1+nabc

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*nabc(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*nabc(i,:,j,1)
  enddo
enddo

do j=1,spy
  do i=1,spx
    a2(i,:,j,1)=-ky(j+cstart(3))*nabc(i,:,j,2)
    a2(i,:,j,2)=ky(j+cstart(3))*nabc(i,:,j,1)
  enddo
enddo

call dz(nabc,a3)

deallocate(nabc)

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

call spectral_to_phys(a1,a4f,1)
call spectral_to_phys(a2,a5f,1)
call spectral_to_phys(a3,a6f,1)

a1f=1d0-phi**(2d0)

!!NORMALIZATION OF THE PHI GRADIENT
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2d0+phiny(i,k,j)**2d0+phinz(i,k,j)**2d0)
      phinx(i,k,j)=phinx(i,k,j)/modnabphi
      phiny(i,k,j)=phiny(i,k,j)/modnabphi
      phinz(i,k,j)=phinz(i,k,j)/modnabphi
    enddo
  enddo
enddo

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      normflux=a4f(i,k,j)*phinx(i,k,j)+a5f(i,k,j)*phiny(i,k,j)+a6f(i,k,j)*phinz(i,k,j)
      a4f(i,k,j)=-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)*phinx(i,k,j) + normflux*phinx(i,k,j)
      a5f(i,k,j)=-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)*phiny(i,k,j) + normflux*phiny(i,k,j)
      a6f(i,k,j)=-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)*phinz(i,k,j) + normflux*phinz(i,k,j)
    enddo
  enddo
enddo

deallocate(a1f,phinx,phiny,phinz)

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

! COMPUITING THE DIVERGENCE OF THE CORRECTION
allocate(a4(spx,nz,spy,2))
allocate(a5(spx,nz,spy,2))
allocate(a6(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a5(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a5(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a6)

deallocate(a1,a2,a3)

sphi=sphi+(a4+a5+a6)/pe

deallocate(a4,a5,a6)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 3
! PROFILE-CORRECTED DISABLED AT THE WALLS MF!!
! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

allocate(nabc(spx,nz,spy,2))
nabc=sphi
sphi=-(1.0d0+s_coeff)/pe*sphi

!Calculate convective term
allocate(a1f(nx,fpz,fpy))
! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

allocate(phinz(nx,fpz,fpy))
phinz=a1f

call phys_to_spectral(convf,a1,1)

deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1


!CALCULATE THE LAMBDA TERM (EXPLICIT)
a1f=1.0d0-phi**(2.0d0)
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      phinx(i,k,j)=phinx(i,k,j)/modnabphi
      phiny(i,k,j)=phiny(i,k,j)/modnabphi
      phinz(i,k,j)=phinz(i,k,j)/modnabphi
    enddo
  enddo
enddo

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))


do j=1,fpy
  do k=1,fpz
    do i=1,nx
      a4f(i,k,j)=a1f(i,k,j)*phinx(i,k,j)
      a5f(i,k,j)=a1f(i,k,j)*phiny(i,k,j)
      a6f(i,k,j)=a1f(i,k,j)*phinz(i,k,j)
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))
allocate(a5(spx,nz,spy,2))
allocate(a6(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a5(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a5(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a6)

!SUM EVEYTHING TO SPHI
!! DISABLE AT THE WALLS funflux(k)
do k=1,fpz
   funflux(k)=0.5d0*(tanh((abs(z(fstart(2)+k)) - 0.975d0)/0.01d0)+1d0)
!   funflux(k)=0d0
!   if ((fstart(2)+k) .eq. 1)  funflux(k)=1d0
!   if ((fstart(2)+k) .eq. nz) funflux(k)=1d0
!   print*,'RANK',RANK,'funflux',funflux(k)
enddo
!!!
nabc=(lamphi/pe)*nabc-(lamphi/(sqrt(2d0)*ch*pe))*(a4+a5+a6)
deallocate(a4,a5,a6)
allocate(nabcf(nx,fpz,fpy))
call spectral_to_phys(nabc,nabcf,1)
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      nabcf(i,k,j)=(1d0-funflux(k))*nabcf(i,k,j)
    enddo
  enddo
enddo
call phys_to_spectral(nabcf,nabc,1)
deallocate(nabcf)

sphi=sphi+nabc
deallocate(nabc)

!Calculate nabla^2 \phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3,a1,a1f)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 7
! PROFILE CORRECTION kill the gradients
! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo
sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi

!######## doubleèlus single well potential ############
!allocate(a1f(nx,fpz,fpy))

!call spectral_to_phys(sphi,a1f,1)

!do j=1,fpy
!  do k=1,fpz
!    do i=1,nx
!      a1f(i,k,j)=a1f(i,k,j)*(1.0d0 - p_psisol(i,k,j))
!!      if((a1f(i,k,j).le.0.0001d0).and.(a1f(i,k,j).ge.0.0d0))  write(*,*) "0 to 1", a1f(i,k,j)
!    enddo
!  enddo
!enddo


!call phys_to_spectral(a1f,sphi,0)
!sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi
!a1f=0.0d0
!################################################################à

!Calculate convective term

allocate(a1f(nx,fpz,fpy))
! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

allocate(phinz(nx,fpz,fpy))
phinz=a1f

call phys_to_spectral(convf,a1,1)

deallocate(convf)

! sum all the convective terms to sphi
sphi=sphi-a1


! calculate nabla^2 phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))


call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3)


! calculate profile-corrected flux
a1f=1.0d0-phi**(2.0d0)

epsnum=1.0d0/(50.0d0*Ch)

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      mask=max(modnabphi-epsnum,0d0)/(modnabphi-epsnum)
      a4f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinx(i,k,j)/modnabphi
      a5f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phiny(i,k,j)/modnabphi
      a6f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinz(i,k,j)/modnabphi
      a4f(i,k,j)=a4f(i,k,j)*mask
      a5f(i,k,j)=a5f(i,k,j)*mask
      a6f(i,k,j)=a6f(i,k,j)*mask
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz,a1f)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a2)

!SUM EVEYTHING TO SPHI
sphi=sphi+(lamphi/pe)*(a4+a1+a2)

!######## three-phase #########
!sphi=sphi+ch_psisolc/pe
!#######################
deallocate(a1,a2,a3,a4)
!deallocate(u_les)
!deallocate(v_les)
!deallocate(uc_les)
!deallocate(vc_les)
#endif


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 5
! Flux-Corrected kill the gradients

allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

allocate(nabc(spx,nz,spy,2))
nabc=ch**2.0d0*sphi
sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi

!Calculate convective term
allocate(a1f(nx,fpz,fpy))

! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)
allocate(phinx(nx,fpz,fpy))
phinx=a1f

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)
allocate(phiny(nx,fpz,fpy))
phiny=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)
allocate(phinz(nx,fpz,fpy))
phinz=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

call phys_to_spectral(convf,a1,1)
deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1

! computing 1/Pe nabla^2 phi^3
a1f=phi**(3d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3)

!COMPUTING THE DIVERGENCE TERM..
nabc=-a1+phic+nabc

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*nabc(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*nabc(i,:,j,1)
  enddo
enddo

do j=1,spy
  do i=1,spx
    a2(i,:,j,1)=-ky(j+cstart(3))*nabc(i,:,j,2)
    a2(i,:,j,2)=ky(j+cstart(3))*nabc(i,:,j,1)
  enddo
enddo

call dz(nabc,a3)

deallocate(nabc)

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

call spectral_to_phys(a1,a4f,1)
call spectral_to_phys(a2,a5f,1)
call spectral_to_phys(a3,a6f,1)

a1f=1d0-phi**(2d0)

!!NORMALIZATION OF THE PHI GRADIENT
epsnum=1.0d0/(50.0d0*Ch)


do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2d0+phiny(i,k,j)**2d0+phinz(i,k,j)**2d0)
      mask=max(modnabphi-epsnum,0d0)/(modnabphi-epsnum)
      normflux=(a4f(i,k,j)*phinx(i,k,j)+a5f(i,k,j)*phiny(i,k,j)+a6f(i,k,j)*phinz(i,k,j))/modnabphi
      a4f(i,k,j)=+(-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)+normflux)*phinx(i,k,j)/modnabphi
      a5f(i,k,j)=+(-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)+normflux)*phiny(i,k,j)/modnabphi
      a6f(i,k,j)=+(-lamphi/(sqrt(2d0)*ch)*a1f(i,k,j)+normflux)*phinz(i,k,j)/modnabphi
      a4f(i,k,j)=a4f(i,k,j)*mask
      a5f(i,k,j)=a5f(i,k,j)*mask
      a6f(i,k,j)=a6f(i,k,j)*mask
    enddo
  enddo
enddo

deallocate(a1f,phinx,phiny,phinz)

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

! COMPUITING THE DIVERGENCE OF THE CORRECTION
allocate(a4(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a2)



sphi=sphi+(a4+a1+a2)/pe

deallocate(a1,a2,a3,a4)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


#if phicorflag == 6
! Calculating chem. potential= phi^3 - phi*(1- psi_sol)
! PROFILE CORRECTION kill the gradients
! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))
! Multiplication phi*(1-psi_sol) in physical

allocate(a1f(nx,fpz,fpy))

call spectral_to_phys(phic,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      a1f(i,k,j)=a1f(i,k,j)*(1.0d0 - p_psisol(i,k,j))
!      if((a1f(i,k,j).le.0.0001d0).and.(a1f(i,k,j).ge.0.0d0))  write(*,*) "0 to 1", a1f(i,k,j)
    enddo
  enddo
enddo
call phys_to_spectral(a1f,sphi,0)


call dz(sphi,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo
sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi

a1f=0.0d0
!################################################################à

!Calculate convective term

!allocate(a1f(nx,fpz,fpy))
! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

allocate(phinz(nx,fpz,fpy))
phinz=a1f

call phys_to_spectral(convf,a1,1)

deallocate(convf)

! sum all the convective terms to sphi
sphi=sphi-a1


! calculate nabla^2 phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))


call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3)


! calculate profile-corrected flux
a1f=1.0d0-phi**(2.0d0)

epsnum=1.0d0/(50.0d0*Ch)

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      mask=max(modnabphi-epsnum,0d0)/(modnabphi-epsnum)
      a4f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinx(i,k,j)/modnabphi
      a5f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phiny(i,k,j)/modnabphi
      a6f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinz(i,k,j)/modnabphi
      a4f(i,k,j)=a4f(i,k,j)*mask
      a5f(i,k,j)=a5f(i,k,j)*mask
      a6f(i,k,j)=a6f(i,k,j)*mask
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz,a1f)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a2)

!SUM EVEYTHING TO SPHI
sphi=sphi+(lamphi/pe)*(a4+a1+a2)
!######### normal unit vector of the solid phase ############
allocate(a1f(nx,fpz,fpy))
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*p_psisolc(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,1)
allocate(psisolnx(nx,fpz,fpy))
psisolnx=a1f
! respect y
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*p_psisolc(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,1)
allocate(psisolny(nx,fpz,fpy))
psisolny=a1f

! respect z
call dz(p_psisolc,a1)
call spectral_to_phys(a1,a1f,1)
allocate(psisolnz(nx,fpz,fpy))
psisolnz=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabpsisol=sqrt(psisolnx(i,k,j)**2+psisolny(i,k,j)**2&
      +psisolnz(i,k,j)**2)
      psisolnx(i,k,j)=psisolnx(i,k,j)/modnabpsisol
      psisolny(i,k,j)=psisolny(i,k,j)/modnabpsisol
      psisolnz(i,k,j)=psisolnz(i,k,j)/modnabpsisol
    enddo
  enddo
enddo
deallocate(a1f)
! taking the tensor product between the normal unit vectors
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      psisolnx(i,k,j)=psisolnx(i,k,j)**2
      psisolny(i,k,j)=psisolny(i,k,j)**2
      psisolnz(i,k,j)=psisolnz(i,k,j)**2
    enddo
  enddo
enddo
! taking the gradient of this solution
call phys_to_spectral(psisolnx,a1,0)
call phys_to_spectral(psisolny,a2,0)
call phys_to_spectral(psisolnz,a3,0)

call dz_fg(a3,a4)

do j=1,spy
  do k=1,nz
    do i=1,spx
      a4(i,k,j,1)=a4(i,k,j,1)-kx(i+cstart(1))*a1(i,k,j,2) &
                           & -ky(j+cstart(3))*a2(i,k,j,2)
      a4(i,k,j,2)=a4(i,k,j,2)+kx(i+cstart(1))*a1(i,k,j,1) &
                           & +ky(j+cstart(3))*a2(i,k,j,1)
    enddo
  enddo
enddo

allocate(a4f(nx,fpz,fpy))
call spectral_to_phys(a4,a4f,1)
!######## Interfacial energies to control wetability #########
!First we define the interfacial energies of the fluid particle: (1+phi-3*psi)
allocate(a1f(nx,fpz,fpy))

a1f=0.5*(1+phi-3*p_psisol)
call phys_to_spectral(a1f,a1,0)
! Laplacian of the term
call dz(a1,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo
!taking the laplacian of a3
call dz(a3,a4)
call dz(a4,a1)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=a1(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,1)
    a1(i,:,j,2)=a1(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,2)
  enddo
enddo
call spectral_to_phys(a1,a1f,1)

!a1f=(1.0d0-a4f)*a1f
a1f=a1f-a4f*a1f

call phys_to_spectral(a1f,a1,0)
sphi=sphi-a1*0.5*(eta_fp**2)/pe
!Secondly we define the interfacial energies of the drop particle: (1+phi-3*psi)
a1f=0.0d0
a1=0.0d0
a2=0.0d0
a3=0.0d0
a4=0.0d0
a1f=0.5*(1-phi-3*p_psisol)

call dz(a1,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo
!taking the laplacian of a3
call dz(a3,a4)
call dz(a4,a1)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=a1(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,1)
    a1(i,:,j,2)=a1(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,2)
  enddo
enddo
call spectral_to_phys(a1,a1f,1)

!a1f=(1.0d0-a4f)*a1f
a1f=a1f-a4f*a1f
call phys_to_spectral(a1f,a1,1)
sphi=sphi-a1*0.5*(eta_dp**2)/pe

!#######################
deallocate(a1f)
deallocate(a4f)
deallocate(a1,a2,a3,a4)
deallocate(psisolnx,psisolny,psisolnz)
#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 4
! Calculating f_0= 0.25*phi^4 - 0.5*phi^2 + 0.5*(phi^2+1)psi_sol
! mu_0= phi^3/Pe - (s+1)phi^2/Pe + phi*psi_sol/pe
! PROFILE CORRECTION kill the gradients
! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))
! Multiplication phi*(1-psi_sol) in physical

allocate(a1f(nx,fpz,fpy))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo
!-----------------------------------------
sphi=-(1.0d0+s_coeff-lamphi)/pe*sphi

!##################################################################
!##########  Coupling term with the solid #########################

call spectral_to_phys(phic,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
!      if(p_psisol(i,k,j).gt.0.0d0)then
      a1f(i,k,j)=coef_ternary*(a1f(i,k,j)-wet_par)*p_psisol(i,k,j)
!      endif
    enddo
  enddo
enddo
! laplacian of psi_sol*phi
allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))
call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)

do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo

sphi=sphi + a3/pe
deallocate(a2)
deallocate(a3)
a1f=0.0d0
!##################################################################
!Calculate convective term

!allocate(a1f(nx,fpz,fpy))
! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)
    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)
    enddo
  enddo
enddo

allocate(phinz(nx,fpz,fpy))
phinz=a1f

call phys_to_spectral(convf,a1,1)

deallocate(convf)

! sum all the convective terms to sphi
sphi=sphi-a1


! calculate nabla^2 phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))


call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3)


! calculate profile-corrected flux
a1f=1.0d0-phi**(2.0d0)

epsnum=1.0d0/(50.0d0*Ch)

 allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      mask=max(modnabphi-epsnum,0d0)/(modnabphi-epsnum)
      a4f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinx(i,k,j)/modnabphi
      a5f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phiny(i,k,j)/modnabphi
      a6f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinz(i,k,j)/modnabphi
      a4f(i,k,j)=a4f(i,k,j)*mask
      a5f(i,k,j)=a5f(i,k,j)*mask
      a6f(i,k,j)=a6f(i,k,j)*mask
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz)
deallocate(a1f)
allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a2)

!SUM EVEYTHING TO SPHI
!sphi=sphi+(lamphi/pe)*(a4+a1+a2)

!######## Interfacial energies to control wetability #########
!First we define the interfacial energies of the fluid particle: (1-phi-2*psi)
allocate(a1f(nx,fpz,fpy))

!######################################## interfacial ###############

a1f=0.0d0
a1=0.0d0
a2=0.0d0
a3=0.0d0
a4=0.0d0

a1f=p_psisol


call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo
!taking the laplacian of a3
call dz(a3,a4)
call dz(a4,a1)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=a1(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,1)
    a1(i,:,j,2)=a1(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,2)
  enddo
enddo
!sphi=sphi-a1*(eta_sol**(2.0d0))/pe

a1f=0.0d0
a1=0.0d0
a2=0.0d0
a3=0.0d0
a4=0.0d0
do j=1,fpy
  do k=1,fpz
    do i=1,nx
!if(p_psisol(i,k,j).gt.0.1d0) then
  a1f(i,k,j)=0.5*(1.0d0-phi(i,k,j)-2.0d0*p_psisol(i,k,j))
 ! else
 !  a1f(i,k,j)=0.0d0
 !endif
    enddo
  enddo
enddo

call phys_to_spectral(a1f,a1,1)
! Laplacian of the term
call dz(a1,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo
!taking the laplacian of a3
call dz(a3,a4)
call dz(a4,a1)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=a1(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,1)
    a1(i,:,j,2)=a1(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,2)
  enddo
enddo

!sphi=sphi+a1*0.5*(eta_fp**(2.0d0))/pe

!Secondly we define the interfacial energies of the drop particle: (1+phi-2*psi)
a1f=0.0d0
a1=0.0d0
a2=0.0d0
a3=0.0d0
a4=0.0d0
do j=1,fpy
  do k=1,fpz
    do i=1,nx
! if(p_psisol(i,k,j).gt.0.1d0) then
a1f(i,k,j)=0.5*(1.0d0+phi(i,k,j)-2.0d0*p_psisol(i,k,j))
 ! else
 ! a1f(i,k,j)=0.0d0
! endif
    enddo
  enddo
enddo

call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo
!taking the laplacian of a3
call dz(a3,a4)
call dz(a4,a1)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=a1(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,1)
    a1(i,:,j,2)=a1(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a3(i,:,j,2)
  enddo
enddo
!sphi=sphi-a1*0.5*(eta_dp**(2.0d0))/pe

!#######################

!######### normal unit vector of the solid phase ############

do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*p_psisolc(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,1)
allocate(psisolnx(nx,fpz,fpy))
psisolnx=a1f
! respect y
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*p_psisolc(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,1)
allocate(psisolny(nx,fpz,fpy))
psisolny=a1f

! respect z
call dz(p_psisolc,a1)
call spectral_to_phys(a1,a1f,1)
allocate(psisolnz(nx,fpz,fpy))
psisolnz=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
 !if(p_psisol(i,k,j).gt.0.0d0) then
      modnabpsisol=sqrt(psisolnx(i,k,j)**2+psisolny(i,k,j)**2&
      +psisolnz(i,k,j)**2)
      psisolnx(i,k,j)=psisolnx(i,k,j)/modnabpsisol
      psisolny(i,k,j)=psisolny(i,k,j)/modnabpsisol
      psisolnz(i,k,j)=psisolnz(i,k,j)/modnabpsisol
 ! else
 !       psisolnx(i,k,j)=0.0d0
 !       psisolny(i,k,j)=0.0d0
 !       psisolnz(i,k,j)=0.0d0
 ! endif
    enddo
  enddo
enddo

! getting the nabla mu= (nalba phi^3 + nabla phi + nabla (phi*psisol))/pe
!nalba phi^3
allocate(gradmux(spx,nz,spy,2))
allocate(gradmuy(spx,nz,spy,2))
allocate(gradmuz(spx,nz,spy,2))
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)
call dz(a1,gradmuz)

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=-kx(i+cstart(1))*a1(i,k,j,2)
      gradmux(i,k,j,2)=+kx(i+cstart(1))*a1(i,k,j,1)
      gradmuy(i,k,j,1)=-ky(j+cstart(3))*a1(i,k,j,2)
      gradmuy(i,k,j,2)=+ky(j+cstart(3))*a1(i,k,j,1)
    enddo
  enddo
enddo
!nabla phi
a2=phic*(1.0d0+s_coeff-lamphi)
call dz(a2,a1)
gradmuz=gradmuz-a1

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)+kx(i+cstart(1))*phic(i,k,j,2)*(1.0d0+s_coeff-lamphi)
      gradmux(i,k,j,2)=gradmux(i,k,j,2)-kx(i+cstart(1))*phic(i,k,j,1)*(1.0d0+s_coeff-lamphi)
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)+ky(j+cstart(3))*phic(i,k,j,2)*(1.0d0+s_coeff-lamphi)
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)-ky(j+cstart(3))*phic(i,k,j,1)*(1.0d0+s_coeff-lamphi)
    enddo
  enddo
enddo

!nabla (phi*psisol)
call spectral_to_phys(phic,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      !if(p_psisol(i,k,j).gt.0.0d0) then
      a1f(i,k,j)=coef_ternary*(a1f(i,k,j)-wet_par)*p_psisol(i,k,j)
      !endif
    enddo
  enddo
enddo

call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
gradmuz=gradmuz+a2

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)-kx(i+cstart(1))*a1(i,k,j,2)
      gradmux(i,k,j,2)=gradmux(i,k,j,2)+kx(i+cstart(1))*a1(i,k,j,1)
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)-ky(j+cstart(3))*a1(i,k,j,2)
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)+ky(j+cstart(3))*a1(i,k,j,1)
    enddo
  enddo
enddo
!-ch^2 nabla^2 phi
call dz(phic,a2)
call dz(a2,a3)

do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

call dz(a3,a1)
gradmuz=gradmuz-(ch**2.0d0)*a1

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)+kx(i+cstart(1))*a3(i,k,j,2)*(ch**(2.0d0))
      gradmux(i,k,j,2)=gradmux(i,k,j,2)-kx(i+cstart(1))*a3(i,k,j,1)*(ch**(2.0d0))
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)+ky(j+cstart(3))*a3(i,k,j,2)*(ch**(2.0d0))
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)-ky(j+cstart(3))*a3(i,k,j,1)*(ch**(2.0d0))
    enddo
  enddo
enddo
!#####
!taking to physical space
allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))
call spectral_to_phys(gradmux,a4f,1)
call spectral_to_phys(gradmuy,a5f,1)
call spectral_to_phys(gradmuz,a6f,1)

! taking the tensor product between the normal unit vectors
allocate(tenif(nx,fpz,fpy))
allocate(tenjf(nx,fpz,fpy))
allocate(tenkf(nx,fpz,fpy))

tenif=0.d0
tenjf=0.d0
tenkf=0.d0

do j=1,fpy
  do k=1,fpz
    do i=1,nx
!    if(p_psisol(i,k,j).gt.0.0d0) then
    tenif(i,k,j)=p_psisol(i,k,j)*(a4f(i,k,j)*psisolnx(i,k,j)**2 + &
    &a5f(i,k,j)*psisolnx(i,k,j)*psisolny(i,k,j) + &
    & a6f(i,k,j)*psisolnx(i,k,j)*psisolnz(i,k,j))
    tenjf(i,k,j)=p_psisol(i,k,j)*(a4f(i,k,j)*psisolny(i,k,j)*psisolnx(i,k,j) &
    & + a5f(i,k,j)*psisolny(i,k,j)**2+ a6f(i,k,j)*psisolny(i,k,j)*psisolnz(i,k,j))
    tenkf(i,k,j)=p_psisol(i,k,j)*(a4f(i,k,j)*psisolnx(i,k,j)*psisolnz(i,k,j) +&
    &  a5f(i,k,j)*psisolny(i,k,j)*psisolnz(i,k,j) + a6f(i,k,j)*psisolnz(i,k,j)**2)
!    endif
    enddo
  enddo
enddo
!! taking the gradient of this solution
call phys_to_spectral(tenif,a1,0)
call phys_to_spectral(tenjf,a2,0)
call phys_to_spectral(tenkf,a3,0)

call dz_fg(a3,a4)

do j=1,spy
  do k=1,nz
    do i=1,spx
      a4(i,k,j,1)=a4(i,k,j,1)-kx(i+cstart(1))*a1(i,k,j,2) &
                           & -ky(j+cstart(3))*a2(i,k,j,2)
      a4(i,k,j,2)=a4(i,k,j,2)+kx(i+cstart(1))*a1(i,k,j,1) &
                           & +ky(j+cstart(3))*a2(i,k,j,1)
    enddo
  enddo
enddo

sphi=sphi-a4/pe

deallocate(a1f)
deallocate(a4f,a5f,a6f)

deallocate(a1,a2,a3,a4)
deallocate(psisolnx,psisolny,psisolnz)
deallocate(tenif,tenjf,tenkf)
deallocate(gradmux,gradmuy,gradmuz)
#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#if phicorflag == 0
! STANDARD MODEL

! calculate -(1+s)/Pe*nabla^2 phi
allocate(a1(spx,nz,spy,2))

call dz(phic,a1)
call dz(a1,sphi)

do j=1,spy
  do i=1,spx
    sphi(i,:,j,1)=sphi(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    sphi(i,:,j,2)=sphi(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

sphi=-(1.0d0+s_coeff)/pe*sphi
!##################################################################
!################# half time artifitially cut ######################à
if(cho_sub_t.eq.2) then
!##########  Coupling term with the solid #########################
allocate(a1f(nx,fpz,fpy))
a1f=0.0d0
call spectral_to_phys(phic,a1f,1)
cut_flux=1.0d0
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      if(p_psisol(i,k,j).lt.0.50d0) then
      cut_flux=p_psisol(i,k,j)
      else
      cut_flux=1.0d0
      endif

      a1f(i,k,j)=coef_ternary*(a1f(i,k,j)-wet_par)*(p_psisol(i,k,j)*cut_flux)

    enddo
  enddo
enddo
! laplacian of psi_sol*phi
allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))
call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
call dz(a2,a3)

do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,:,j,2)
  enddo
enddo

! !######## new ###########
call spectral_to_phys(a3,a1f,1)
!
do j=1,fpy
  do k=1,fpz
    do i=1,nx
      if(p_psisol(i,k,j).lt.0.50d0) then
      cut_flux=p_psisol(i,k,j)
      else
      cut_flux=1.0d0
      endif
      a1f(i,k,j)=a1f(i,k,j)*(cut_flux**8.0d0)

    end do
  end do
end do
call phys_to_spectral(a1f,a3,1)
!###########
sphi=sphi + a3/pe
deallocate(a2)
deallocate(a3)
deallocate(a1f)

endif
!##################################################################
!Calculate convective term
allocate(a1f(nx,fpz,fpy))

! u*(d phi /dx)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo
call spectral_to_phys(uc,u,1)
call spectral_to_phys(a1,a1f,1)

allocate(convf(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
!      if(p_psisol(i,k,j).lt.0.005d0)then
      convf(i,k,j)=a1f(i,k,j)*u(i,k,j)!*(1d0-p_psisol(i,k,j))

    enddo
  enddo
enddo

allocate(phinx(nx,fpz,fpy))
phinx=a1f

! v*(d phi /dy)
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo
call spectral_to_phys(vc,v,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
!      if(p_psisol(i,k,j).lt.0.005d0)then
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*v(i,k,j)!*(1d0-p_psisol(i,k,j))
!      else
!      convf(i,k,j)=0.0d0
!      endif
    enddo
  enddo
enddo

allocate(phiny(nx,fpz,fpy))
phiny=a1f

! w*(d phi /dz)
call dz(phic,a1)

call spectral_to_phys(wc,w,1)
call spectral_to_phys(a1,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
    !if(p_psisol(i,k,j).lt.0.005d0)then
      convf(i,k,j)=convf(i,k,j)+a1f(i,k,j)*w(i,k,j)!*(1d0-p_psisol(i,k,j))
    !else
    !convf(i,k,j)=0.0d0
    !endif
    enddo
  enddo
enddo
allocate(phinz(nx,fpz,fpy))
phinz=a1f
call phys_to_spectral(convf,a1,1)

deallocate(convf)
! sum all the convective terms to sphi
sphi=sphi-a1

!Calculate nabla^2 \phi^3
a1f=phi**(3.0d0)

call phys_to_spectral(a1f,a1,1)

allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call dz(a1,a2)
call dz(a2,a3)

deallocate(a2)

do j=1,spy
  do k=1,nz
    do i=1,spx
      sphi(i,k,j,1)=sphi(i,k,j,1)+(a3(i,k,j,1)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,1))/pe
      sphi(i,k,j,2)=sphi(i,k,j,2)+(a3(i,k,j,2)-k2(i+cstart(1),j+cstart(3))*a1(i,k,j,2))/pe
    enddo
  enddo
enddo

deallocate(a3,a1,a1f)

!! calculate profile-corrected flux
if(cho_sub_t.eq.2) then
allocate(a1f(nx,fpz,fpy))

a1f=1.0d0-phi**(2.0d0)

epsnum=1.0d0/(50.0d0*Ch)

allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      modnabphi=sqrt(phinx(i,k,j)**2+phiny(i,k,j)**2+phinz(i,k,j)**2)
      mask=max(modnabphi-epsnum,0d0)/(modnabphi-epsnum)
      a4f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinx(i,k,j)/modnabphi
      a5f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phiny(i,k,j)/modnabphi
      a6f(i,k,j)=-1.0d0/(sqrt(2.0d0)*Ch)*a1f(i,k,j)*phinz(i,k,j)/modnabphi
      a4f(i,k,j)=a4f(i,k,j)*mask
      a5f(i,k,j)=a5f(i,k,j)*mask
      a6f(i,k,j)=a6f(i,k,j)*mask
    enddo
  enddo
enddo

deallocate(phinx,phiny,phinz,a1f)
allocate(a1(spx,nz,spy,2))
allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))

call phys_to_spectral(a4f,a1,1)
call phys_to_spectral(a5f,a2,1)
call phys_to_spectral(a6f,a3,1)

deallocate(a4f,a5f,a6f)

allocate(a4(spx,nz,spy,2))

!X DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a4(i,:,j,1)=-kx(i+cstart(1))*a1(i,:,j,2)
    a4(i,:,j,2)=kx(i+cstart(1))*a1(i,:,j,1)
  enddo
enddo
!Y DERIVATIVE OF CORRECTION
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*a2(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*a2(i,:,j,1)
  enddo
enddo
!Z DERIVATIVE OF CORRECTION
call dz(a3,a2)

!SUM EVEYTHING TO SPHI
sphi=sphi+(lamphi/pe)*(a4+a1+a2)

deallocate(a1,a2,a3,a4)
endif!flux corrected
!!flux cut bounded for the solid phase_field
!!######### normal unit vector of the solid phase ############
allocate(a1(spx,nz,spy,2))
allocate(a1f(nx,fpz,fpy))

do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-kx(i+cstart(1))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=kx(i+cstart(1))*p_psisolc(i,:,j,1)
!shinto
    ! a1(i,:,j,1)=-kx(i+cstart(1))*phic(i,:,j,2)
    ! a1(i,:,j,2)=kx(i+cstart(1))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,0)
allocate(psisolnx(nx,fpz,fpy))
psisolnx=a1f
! respect y
do j=1,spy
  do i=1,spx
    a1(i,:,j,1)=-ky(j+cstart(3))*p_psisolc(i,:,j,2)
    a1(i,:,j,2)=ky(j+cstart(3))*p_psisolc(i,:,j,1)
!shinto
    ! a1(i,:,j,1)=-ky(j+cstart(3))*phic(i,:,j,2)
    ! a1(i,:,j,2)=ky(j+cstart(3))*phic(i,:,j,1)
  enddo
enddo

call spectral_to_phys(a1,a1f,0)
allocate(psisolny(nx,fpz,fpy))
psisolny=a1f

! respect z
call dz(p_psisolc,a1)
!shinto
! call dz(phic,a1)

call spectral_to_phys(a1,a1f,1)
allocate(psisolnz(nx,fpz,fpy))
psisolnz=a1f

do j=1,fpy
  do k=1,fpz
    do i=1,nx
       modnabpsisol=sqrt(psisolnx(i,k,j)**2+psisolny(i,k,j)**2&
      +psisolnz(i,k,j)**2)
      psisolnx(i,k,j)=psisolnx(i,k,j)/modnabpsisol
      psisolny(i,k,j)=psisolny(i,k,j)/modnabpsisol
      psisolnz(i,k,j)=psisolnz(i,k,j)/modnabpsisol

    enddo
  enddo
enddo

deallocate(a1)
deallocate(a1f)
! getting the nabla mu= (nalba phi^3 + nabla phi + nabla (phi*psisol))/pe
!nalba phi^3
allocate(gradmux(spx,nz,spy,2))
allocate(gradmuy(spx,nz,spy,2))
allocate(gradmuz(spx,nz,spy,2))
allocate(a1f(nx,fpz,fpy))
allocate(a1(spx,nz,spy,2))
allocate(a2(spx,nz,spy,2))
allocate(a3(spx,nz,spy,2))
allocate(a4(spx,nz,spy,2))

a1f=phi**(3.0d0)
call phys_to_spectral(a1f,a1,1)
call dz(a1,gradmuz)

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=-kx(i+cstart(1))*a1(i,k,j,2)
      gradmux(i,k,j,2)=+kx(i+cstart(1))*a1(i,k,j,1)
      gradmuy(i,k,j,1)=-ky(j+cstart(3))*a1(i,k,j,2)
      gradmuy(i,k,j,2)=+ky(j+cstart(3))*a1(i,k,j,1)
    enddo
  enddo
enddo

!nabla (phi*psisol)
call spectral_to_phys(phic,a1f,1)

do j=1,fpy
  do k=1,fpz
    do i=1,nx
      a1f(i,k,j)=coef_ternary*(a1f(i,k,j)-wet_par)*p_psisol(i,k,j)
    enddo
  enddo
enddo

call phys_to_spectral(a1f,a1,1)

call dz(a1,a2)
gradmuz=gradmuz+a2

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)-kx(i+cstart(1))*a1(i,k,j,2)
      gradmux(i,k,j,2)=gradmux(i,k,j,2)+kx(i+cstart(1))*a1(i,k,j,1)
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)-ky(j+cstart(3))*a1(i,k,j,2)
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)+ky(j+cstart(3))*a1(i,k,j,1)
    enddo
  enddo
enddo

! calculate -(1+s)/Pe*nabla phi

a2=phic
call dz(a2,a1)
gradmuz=gradmuz-a1!*(1.0d0+s_coeff)

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)+kx(i+cstart(1))*phic(i,k,j,2)!*(1.0d0+s_coeff)
      gradmux(i,k,j,2)=gradmux(i,k,j,2)-kx(i+cstart(1))*phic(i,k,j,1)!*(1.0d0+s_coeff)
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)+ky(j+cstart(3))*phic(i,k,j,2)!*(1.0d0+s_coeff)
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)-ky(j+cstart(3))*phic(i,k,j,1)!*(1.0d0+s_coeff)
    enddo
  enddo
enddo

! !Interfacial term
!
! Laplacian of the term
call dz(phic,a2)
call dz(a2,a3)
do j=1,spy
  do i=1,spx
    a3(i,:,j,1)=a3(i,:,j,1)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,1)
    a3(i,:,j,2)=a3(i,:,j,2)-k2(i+cstart(1),j+cstart(3))*phic(i,:,j,2)
  enddo
enddo

! gradient of nabla^2 phi
call dz(a3,a1)
gradmuz=gradmuz-(ch**2d0)*a1

do j=1,spy
  do k=1,nz
    do i=1,spx
      gradmux(i,k,j,1)=gradmux(i,k,j,1)+(ch**2d0)*kx(i+cstart(1))*a3(i,k,j,2)!
      gradmux(i,k,j,2)=gradmux(i,k,j,2)-(ch**2d0)*kx(i+cstart(1))*a3(i,k,j,1)!
      gradmuy(i,k,j,1)=gradmuy(i,k,j,1)+(ch**2d0)*ky(j+cstart(3))*a3(i,k,j,2)!
      gradmuy(i,k,j,2)=gradmuy(i,k,j,2)-(ch**2d0)*ky(j+cstart(3))*a3(i,k,j,1)!
    enddo
  enddo
enddo

!taking to physical space
allocate(a4f(nx,fpz,fpy))
allocate(a5f(nx,fpz,fpy))
allocate(a6f(nx,fpz,fpy))
call spectral_to_phys(gradmux,a4f,1)
call spectral_to_phys(gradmuy,a5f,1)
call spectral_to_phys(gradmuz,a6f,1)

! taking the tensor product between the normal unit vectors
allocate(tenif(nx,fpz,fpy))
allocate(tenjf(nx,fpz,fpy))
allocate(tenkf(nx,fpz,fpy))

!------
tenif=0.0d0
tenjf=0.0d0
tenkf=0.0d0

do j=1,fpy
  do k=1,fpz
    do i=1,nx
    ! if(p_psisol(i,k,j).gt.0.0d0)then
    tenif(i,k,j)=(a4f(i,k,j)*(psisolnx(i,k,j)**2) + &
    &a5f(i,k,j)*psisolnx(i,k,j)*psisolny(i,k,j) + &
    & a6f(i,k,j)*psisolnx(i,k,j)*psisolnz(i,k,j))
    tenjf(i,k,j)=(a4f(i,k,j)*psisolny(i,k,j)*psisolnx(i,k,j) &
    & + a5f(i,k,j)*(psisolny(i,k,j)**2)+ a6f(i,k,j)*psisolny(i,k,j)*psisolnz(i,k,j))
    tenkf(i,k,j)=(a4f(i,k,j)*psisolnx(i,k,j)*psisolnz(i,k,j) +&
    &  a5f(i,k,j)*psisolny(i,k,j)*psisolnz(i,k,j) + a6f(i,k,j)*(psisolnz(i,k,j)**2))
  ! else
  !   tenif(i,k,j)=0.0d0
  !   tenjf(i,k,j)=0.0d0
  !   tenkf(i,k,j)=0.0d0
  ! endif
!shinto npnp
! tenif(i,k,j)=a4f(i,k,j)*(psisolnx(i,k,j)**2)
! tenjf(i,k,j)=a5f(i,k,j)*(psisolny(i,k,j)**2)
! tenkf(i,k,j)=a6f(i,k,j)*(psisolnz(i,k,j)**2)
    enddo
  enddo
enddo
!! taking the gradient of this solution
call phys_to_spectral(tenif,a1,1)
call phys_to_spectral(tenjf,a2,1)
call phys_to_spectral(tenkf,a3,1)

call dz(a3,a4)

do j=1,spy
  do k=1,nz
    do i=1,spx
      a4(i,k,j,1)=a4(i,k,j,1)-kx(i+cstart(1))*a1(i,k,j,2) &
                           & -ky(j+cstart(3))*a2(i,k,j,2)
      a4(i,k,j,2)=a4(i,k,j,2)+kx(i+cstart(1))*a1(i,k,j,1) &
                           & +ky(j+cstart(3))*a2(i,k,j,1)
    enddo
  enddo
enddo
if(cho_sub_t.eq.8) then
sphi=sphi!-a4/pe
endif
deallocate(a1f)
deallocate(a4f,a5f,a6f)

deallocate(a1,a2,a3,a4)
deallocate(psisolnx,psisolny,psisolnz)
deallocate(tenif,tenjf,tenkf)
deallocate(gradmux,gradmuy,gradmuz)

#endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

return
end
